package main

import (
	"encoding/base64"
	"fmt"
	"net/http"
	"os"
	"strconv"
	"sync"
	"time"

	"Crazy_Push/models/ab"
	"Crazy_Push/models/dblogic"
	"Crazy_Push/models/tables"

	"Crazy_Push/models/log"
)

const (
	//LOGINURL   = "http://testapi.superant.cc/api/login.php"
	LOGINURL   = "http://testapi.superant.cc/api/login.php"
	GATEWAYURL = "http://testapi.superant.cc/api/gateway.php"
	CONFIGURL  = "http://127.0.0.1:8089/api/config.php"
	TESTURL    = "http://testhttp-crazyfox.superant.cc/api/gateway.php"
)

var (
	header     = make(http.Header)
	okNum  int = 0
	badNum int = 0
	reqUrl string
)

func main() {
	args := os.Args
	var limit int
	header.Add("content-type", "application/x-www-form-urlencoded")
	if len(args) < 2 {
		limit = 10
	} else {
		limit, _ = strconv.Atoi(args[1])
	}
	if limit <= 0 {
		limit = 1
	}
	if len(args) < 3 || (args[2] != "login" && args[2] != "gateway" && args[2] != "config") {
		reqUrl = LOGINURL
	} else {
		if args[2] == "login" {
			reqUrl = LOGINURL
		} else if args[2] == "config" {
			reqUrl = CONFIGURL
		} else {
			reqUrl = LOGINURL
		}
	}

	var users []*tables.TMobile
	dblogic.M("User").Order("createtime DESC").Limit(limit).Find(&users)
	var params = map[string]string{
		"device":  "Android",
		"lid":     "1",
		"sid":     "1",
		"version": "1.7.7",
	}

	//go loop()
	fmt.Println("请求个数:", limit, ",请求地址:", reqUrl)
	log.LogInfo("abtest", "请求个数:", limit)
	ts := time.Now()
	fmt.Println("开始时间:", ts)
	wg := sync.WaitGroup{}
	for {
		wg.Add(limit)
		for _, v := range users {
			if reqUrl == LOGINURL {
				params["mobile_request"] = base64.StdEncoding.EncodeToString([]byte(v.Macid + "_superant"))
			} else if reqUrl == GATEWAYURL {
				params["uid"] = v.Uid
				params["mod"] = "Goods"
				params["act"] = "list"
				params["service"] = "1"
			} else if reqUrl == CONFIGURL {
				params["version"] = "1.8.4.2"
				params["sid"] = "1"
			} else {
				params["gps_adid"] = v.Macid
				params["idfa"] = "idfa"
				params["idfv"] = "idfv"
				params["adid"] = "adid"
				params["tracker_token"] = "tracker"
				params["tracker_name"] = "tracker_name"
				params["app_name"] = "app_name"
				params["activity_kind"] = "activity_kind"
				params["created_at"] = "created_at"
				params["event_token"] = "event"
				params["event_name"] = "event_name"
				params["network_name"] = "network_name"
				params["campaign_name"] = "campaign_name"
				params["adgroup_name"] = "adgroup_name"
				params["creative_name"] = "creative_name"
				params["label"] = "label1"
			}
			paramStr := ab.ParseUrlEncode(params)
			if reqUrl == CONFIGURL || reqUrl == TESTURL {
				go getRequest(paramStr, &wg)
			} else {
				go postRequest(paramStr, &wg)
			}
			//chanSend <- paramStr
		}
		wg.Wait()
		cost := time.Since(ts)
		fmt.Println("总共耗时:", cost)
		time.Sleep(time.Second * 3)
	}
}

func getRequest(paramStr string, wg *sync.WaitGroup) {
	start := time.Now()
	str, ret := ab.GetRequest(reqUrl, paramStr, header)
	if ret == 200 {
		okNum++
	} else {
		badNum++
	}

	cost := time.Since(start)
	log.LogInfo("abtest", "请求结果:", "长度:", len(str), ",返回状态:", ret, ",耗时:", cost, ",成功数:", okNum, ",失败数:", badNum)
	wg.Done()
}

func postRequest(paramStr string, wg *sync.WaitGroup) {
	start := time.Now()
	str, ret := ab.PostRequest(reqUrl, paramStr, header)
	if ret == 200 {
		okNum++
	} else {
		badNum++
	}

	cost := time.Since(start)
	log.LogInfo("abtest", "请求结果,长度:", len(str), ",返回状态:", ret, ",耗时:", cost, ",成功数:", okNum, ",失败数:", badNum)
	wg.Done()
}

//func loop() {
//    for {
//        select {
//		case params := <-chanSend: // 客户端建立连接
//            start := time.Now()
//            str, ret := ab.PostRequest(URL, params, header)
//            cost := time.Since(start)
//            fmt.Println("获取到了请求结果:", str, ret)
//            log.LogInfo("abtest", "获取到了请求结果:", str, ret, params, ",耗时:",cost)
//		}
//    }
//}
