package controllers

import (
    "strconv"

	"Crazy_Push/cmd/http/conf"
	"github.com/gin-gonic/gin"
)

//失败json数据
func getErrJson(errCode int) gin.H {
    return getMsgJson(errCode, "")
}

//成功json数据
func getSuccessJson(ret string) gin.H {
    return getMsgJson(0, ret)
}

//返回字符串error code
func getStringCode(errCode int) string {
    return strconv.Itoa(errCode)
}

//返回json数据
func getMsgJson(errCode int, ret string) gin.H {
    var msg string
    if errCode == 0 {
        msg = "请求成功"
    } else {
        msg = config.GetErrMsg(errCode) //错误消息
        //if msg == "" {
        //    msg = "未知消息"
        //}
    }
    return gin.H{
        //"cmd": c.Request.URL.Path,
        "code": strconv.Itoa(errCode),
        "msg":  msg,
        "data": ret,
    }
}

//获取GET传参
func getQryParam(params map[string][]string, c *gin.Context) (map[string]interface{}, int) {
    var code int = 0
    ret := map[string]interface{}{}
    for k, v := range params {
        if v[0] == "" {
            return nil, config.REQUEST_PARAM_ERROR
        }
        if len(v) > 1 {
            code, _ = strconv.Atoi(v[1])
        } else {
            code = 0
        }
        if ok := parseParamValByStr(c.Query(k), k, v[0], code, ret); !ok {
            return nil, code
        }
    }
    return ret, 0
}

//获取参数，将请求参数从string转化成对应类型
func getParams(params map[string][]string, c *gin.Context) (map[string]interface{}, int) {
    var code int = 0
    ret := map[string]interface{}{}
    for k, v := range params {
        if v[0] == "" {
            return nil, config.REQUEST_PARAM_ERROR
        }
        if len(v) > 1 {
            code, _ = strconv.Atoi(v[1])
        } else {
            code = 0
        }
        if ok := parseParamValByStr(c.PostForm(k), k, v[0], code, ret); !ok {
            return nil, code
        }
    }
    return ret, 0
}

//获取编码后的请求参数
func getDecodeParams(params map[string][]string, reqParams map[string]string) (map[string]interface{}, int) {
    var code int = 0
    ret := map[string]interface{}{}
    for k, v := range params {
        if v[0] == "" {
            return nil, config.REQUEST_PARAM_ERROR
        }
        if len(v) > 1 {
            code, _ = strconv.Atoi(v[1])
        } else {
            code = 0
        }
        if ok := parseParamValByStr(reqParams[k], k, v[0], code, ret); !ok {
            return nil, code
        }
    }
    return ret, 0
}

//将请求参数转化成需要的类型
func parseParamValByStr(strVal string, k, flag string, code int, ret map[string]interface{}) bool {
    if strVal == "" && code != 0 {
        return false //参数验证未通过
    }
    switch flag {
        case "float64":
            val, err := strconv.ParseFloat(strVal, 64)
            if err != nil && code != 0 {
                return false //参数验证未通过
            }
            if err == nil {
                ret[k] = val
            }
        case "int64", "int32", "int":
            val, err := strconv.Atoi(strVal)
            if err != nil && code != 0 {
                return false //参数验证未通过
            }
            if err == nil {
                ret[k] = int64(val) //默认全部转成int64格式
            }
        case "uint64", "uint32", "uint":
            val, err := strconv.Atoi(strVal)
            if err != nil && code != 0 {
                return false //参数验证未通过
            }
            if val < 0 {
                return false //请求参数必传正整数
            }
            if err == nil {
                ret[k] = int64(val) //默认全部转成int64格式
            }
        case "bool":
            if strVal == "true" {
                ret[k] = true
            } else if strVal != "" {
                ret[k] = false
            }
        default:
            if strVal != "" {
                ret[k] = strVal
            }
            ret[k] = strVal
    }
    return true
}
