package controllers

import (
	"encoding/json"
	"fmt"
	"net/http"

	"Crazy_Push/cmd/statistics/conf"
	"Crazy_Push/models/dblogic"
	"Crazy_Push/models/log"
	"Crazy_Push/models/tables"

	"github.com/gin-gonic/gin"
)
type FbData struct {
	Ad_id               int64 `json:"ad_id"`
	Ad_objective_name   string `json:"ad_objective_name"`
	Adgroup_id          int64 `json:"adgroup_id"`
	Adgroup_name        string `json:"adgroup_name"`
	Campaign_id         int64 `json:"campaign_id"`
	Campaign_name       string `json:"campaign_name"`
	Campaign_group_id   int64 `json:"campaign_group_id"`
	Campaign_group_name string `json:"campaign_group_name"`
	Account_id          int64 `json:"account_id"`
	Is_instagram        bool `json:"is_instagram"`
	Publisher_platform  string `json:"publisher_platform"`
	Platform_position   string `json:"platform_position"`
}
//adjust数据实时回传
func AdjustCallbacks(c *gin.Context) {
	//上报请求必传参数
	params := map[string][]string{
		"adid":          []string{"string", getStringCode(config.REQUEST_PARAM_ERROR)},
		"idfa":          []string{"string"},
		"gps_adid":      []string{"string"},
		"tracker_name":  []string{"string"},
		"network_name":  []string{"string"},
		"campaign_name": []string{"string"},
		"adgroup_name":  []string{"string"},
		"creative_name": []string{"string"},
		"created_at":    []string{"string"},
		"activity_kind": []string{"string"},
		"label":         []string{"string"},
		"idfv":          []string{"string"},
		"tracker_token": []string{"string"},
		"app_name":      []string{"string"},
		"event_token":   []string{"string"},
		"event_name":    []string{"string"},

		"fb_referrer":                          []string{"string"},
		"PhFbInstallReferrerAdId":              []string{"string"},
		"PhFbInstallReferrerAdGroupId":         []string{"string"},
		"PhFbInstallReferrerAdGroupName":       []string{"string"},
		"PhFbInstallReferrerCampaignId":        []string{"string"},
		"PhFbInstallReferrerCampaignName":      []string{"string"},
		"PhFbInstallReferrerCampaignGroupId":   []string{"string"},
		"PhFbInstallReferrerCampaignGroupName": []string{"string"},
		"PhFbInstallReferrerAccountId":         []string{"string"},
		"PhFbInstallReferrerAdObjectiveName":   []string{"string"},
	}
	ret, errId := getQryParam(params, c) //非加密方式传输
	if errId != 0 {
		log.LogError("adjust_report", "params error", errId, ret)
		c.JSON(http.StatusOK, getErrJson(config.SYSTEM_ERROR))
		return
	}
	log.LogInfo("adjust_callback", "params info", errId, ret)
	insertData :=&tables.TAdjustData{
		Adid:         ret["adid"].(string),
		GpsAdid:      ret["gps_adid"].(string),
		Idfa:         ret["idfa"].(string),
		TrackerName:  ret["tracker_name"].(string),
		NetworkName:  ret["network_name"].(string),
		CampaignName: ret["campaign_name"].(string),
		AdgroupName:  ret["adgroup_name"].(string),
		CreativeName: ret["creative_name"].(string),
		CreatedAt:    ret["created_at"].(string),
		ActivityKind: ret["activity_kind"].(string),
		Label:        ret["label"].(string),
		Idfv:         ret["idfv"].(string),
		TrackerToken: ret["tracker_token"].(string),
		AppName:      ret["app_name"].(string),
		EventToken:   ret["event_token"].(string),
		EventName:    ret["event_name"].(string),
	}
	if insertData.NetworkName=="Unattributed"&&insertData.TrackerName=="Unattributed"{
		fb_referrer :=ret["fb_referrer"].(string)
		if fb_referrer!=""{//FB 广告来源新策略
			fbDatas :=new(FbData)
			if err := json.Unmarshal([]byte(fb_referrer),&fbDatas);err != nil{
				log.LogInfo("adjust_callback", "decode err:", err)
			}
			if fbDatas!=nil{
				insertData.CampaignName = fmt.Sprintf("%s(%d)",fbDatas.Campaign_group_name,fbDatas.Campaign_group_id)
				insertData.AdgroupName = fmt.Sprintf("%s(%d)",fbDatas.Campaign_name,fbDatas.Campaign_id)
				insertData.CreativeName = fmt.Sprintf("%s(%d)",fbDatas.Adgroup_name,fbDatas.Adgroup_id)
				insertData.NetworkName = "Facebook"
				insertData.TrackerName = fmt.Sprintf("%s::%s::%s::%s","Facebook",insertData.CampaignName,insertData.AdgroupName,insertData.CreativeName)
			}
		}
	}
	err := dblogic.M("ADJUSTSTA").Create(insertData).Error

	if err != nil {
		log.LogError("adjust_report", "add adjust record error", err, ret)
		c.JSON(http.StatusOK, getErrJson(config.REQUEST_PARAM_ERROR))
		return
	}

	c.JSON(http.StatusOK, getSuccessJson("ok"))
	return
}

//ajdust再归因上报
func AdjustReCallbacks(c *gin.Context) {
	//上报请求必传参数
	params := map[string][]string{
		"adid":          []string{"string", getStringCode(config.REQUEST_PARAM_ERROR)},
		"idfa":          []string{"string"},
		"gps_adid":      []string{"string"},
		"tracker_name":  []string{"string"},
		"network_name":  []string{"string"},
		"campaign_name": []string{"string"},
		"adgroup_name":  []string{"string"},
		"creative_name": []string{"string"},
		"created_at":    []string{"string"},
		"activity_kind": []string{"string"},
		"label":         []string{"string"},
	}
	ret, errId := getQryParam(params, c) //非加密方式传输
	if errId != 0 {
		c.JSON(http.StatusOK, getErrJson(config.SYSTEM_ERROR))
		return
	}

	//没有传label,不是邀请数据
	if ret["label"] == nil || ret["label"].(string) == "" {
		c.JSON(http.StatusOK, getSuccessJson("ok"))
		return
	}

	err := dblogic.M("ADJUSTSTA").Where("adid=?", ret["adid"].(string)).Save(&tables.TAdjustCallbackData{
		Adid:         ret["adid"].(string),
		GpsAdid:      ret["gps_adid"].(string),
		Idfa:         ret["idfa"].(string),
		TrackerName:  ret["tracker_name"].(string),
		NetworkName:  ret["network_name"].(string),
		CampaignName: ret["campaign_name"].(string),
		AdgroupName:  ret["adgroup_name"].(string),
		CreativeName: ret["creative_name"].(string),
		CreatedAt:    ret["created_at"].(string),
		ActivityKind: ret["activity_kind"].(string),
		Label:        ret["label"].(string),
	}).Error

	if err != nil {
		log.LogError("adjust_callback_report", "add adjust callback record error", err, ret)
		c.JSON(http.StatusOK, getErrJson(config.REQUEST_PARAM_ERROR))
		return
	}

	c.JSON(http.StatusOK, getSuccessJson("ok"))
	return
}
