package controllers

import (
    "net/http"
    "strconv"
    "strings"

    "Crazy_Push/models/log"
	"Crazy_Push/cmd/statistics/conf"
	"Crazy_Push/models/statistics"
	"Crazy_Push/models/games"
	"Crazy_Push/models/dblogic"
	"Crazy_Push/models/okeys"

    "github.com/gin-gonic/gin"

	//jsoniter "github.com/json-iterator/go"
	//"github.com/json-iterator/go/extra"
)

//var json = jsoniter.ConfigCompatibleWithStandardLibrary
//func init() {
//    //兼容php json string(float/int)等类型
//    extra.RegisterFuzzyDecoders()
//}

//客户端小游戏数据上报时长统计
func PlayReport(c *gin.Context) {
    param, ok := c.GetPostForm("param")
    //客户端请求必传参数
    params := map[string][]string {
        //"sid":       []string{"int64", getStringCode(config.REQUEST_PARAM_ERROR)},
        //"lid":       []string{"int64", getStringCode(config.REQUEST_PARAM_ERROR)},
        "uid":       []string{"int64", getStringCode(config.REQUEST_PARAM_ERROR)},
        "play_time": []string{"int64", getStringCode(config.REQUEST_PARAM_ERROR)},
        "play_type": []string{"int64", getStringCode(config.REQUEST_PARAM_ERROR)},
    }

    var ret map[string]interface{}
    var errId int
    if !ok {
        ret, errId = getParams(params, c) //非加密方式传输
    } else {
        //解码请求参数map[string]string
        ret, errId = getDecodeParams(params, games.GetReqParams(param))
    }
    if errId != 0 {
        c.JSON(http.StatusOK, getErrJson(errId))
        return
    }

    reports := statistics.NewPlayReports()
    if !reports.CheckType(ret["play_type"].(int64)) {
        c.JSON(http.StatusOK, getMsgJson(config.REQUEST_PARAM_ERROR, "小游戏类型错误"))
        return
    }
    reports.SavePlayStatistic(ret)
    c.JSON(http.StatusOK, getSuccessJson("ok"))
    return
}

//打点事件上报
func EventReport(c *gin.Context) {
    param, ok := c.GetPostForm("param")
    //客户端请求必传参数
    params := map[string][]string {
        "uid":           []string{"string"},
        "event_id":      []string{"string", getStringCode(config.REQUEST_PARAM_ERROR)},
        "serverTime":    []string{"string"}, //服务器时间
        //"data":          []string{"string", getStringCode(config.REQUEST_PARAM_ERROR)},
        "data":          []string{"string"},
        "commonData":    []string{"string"},
        "sid":           []string{"string"},
        "lid":           []string{"string"},
        "is_new_report": []string{"string"},
        "gid":           []string{"string"},
    }

    var ret map[string]interface{}
    var errId int
    if !ok {
        ret, errId = getParams(params, c) //非加密方式传输
    } else {
        //解码请求参数map[string]string
        ret, errId = getDecodeParams(params, games.GetReqParams(param))
    }

    if errId != 0 {
        c.JSON(http.StatusOK, getErrJson(errId))
        return
    }

    //记录10012事件，排查上报格式情况
    if strings.Contains(ret["event_id"].(string), "10012") {
        log.LogInfo("event_adjust_report_log", "10012 info", ret["data"], ret["event_id"])
    }

    eventId := ret["event_id"].(string)
    report := statistics.NewEventReport()
    //漏斗事件上报
    report.FunnelReport(ret, c.ClientIP())

    if report.IsFunnelReport(eventId){
        c.JSON(http.StatusOK, getSuccessJson("ok"))
        return
    }

    //带uid的用户事件
    if report.IsUserReport(eventId) && ret["uid"].(string) == "" {
        c.JSON(http.StatusOK, getErrJson(config.REQUEST_PARAM_ERROR))
        return
    }

    uid := ret["uid"].(string)
    sid := "1"
    if ret["sid"].(string) != "" {
        sid = ret["sid"].(string)
    }

    stepStr := ""
    data := []string{}
    if ret["data"] != nil {
        data = report.ParseData(ret["data"].(string))
    }
    if len(data) > 0 {
        if strings.Contains(data[0], "[") {
            data2 := report.ParseData(data[0])
            if len(data2) > 0 { //重新复制
                stepStr = data2[0]
                data[0] = data2[0]
            }
        } else {
            stepStr = data[0]
        }
    }

    var step int64 = 0
    if stepStr != "" && eventId != statistics.ADVERTISEID {
        stepInt, err := strconv.Atoi(stepStr)
        if err != nil { //上报格式错误
            c.JSON(http.StatusOK, getSuccessJson("ok"))
            return
        }
        step = int64(stepInt)
    }

    //记录adjust上报异常记录
    if eventId == statistics.ADVERTISEID && stepStr == "" {
        log.LogInfo("event_report_log", "adjust report step empty", ret["data"])
    }

    eventIdInt64, _ := strconv.ParseInt(eventId, 10, 64)
    sidInt64, _ := strconv.ParseInt(sid, 10, 64)

    //其他事件
    switch(eventId) {
        case statistics.ADCLICKED, statistics.ADPLAY:
            c.JSON(http.StatusOK, getSuccessJson("ok"))
            return
            //if uid == "" {
            //    c.JSON(http.StatusOK, getSuccessJson("ok"))
            //    return
            //}
            //tmp, _ := dblogic.R("Statistics").HGet(okeys.Report(eventIdInt64, 0), uid)
            //count, err := strconv.Atoi(tmp)
            //if err != nil { //redis数据有错
            //    c.JSON(http.StatusOK, getSuccessJson("ok"))
            //    return
            //}
            //if stepStr != "" {
            //    dblogic.R("Statistics").HSet(okeys.Report(eventIdInt64, 0), uid+"-"+stepStr, count + 1)
            //}
        case statistics.ADVERTISEID:
            if stepStr != "" {
                dblogic.R("Statistics").HSet(okeys.Report(eventIdInt64, sidInt64), uid, stepStr)
            }
        case statistics.NETWORKCONNECTERROR:
            if len(data) < 2 { //结构错误
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            dblogic.R("Statistics").HIncrBy(okeys.NetworkReport(sidInt64), data[0]+"-"+data[1], 1)
        case statistics.LOGINEVENT:
            if len(data) < 1 { //结构错误
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            dblogic.R("Statistics").HIncrBy(okeys.LoginEventSta(sidInt64), data[0], 1)
        case statistics.GIFTEVENT:
            if len(data) < 2 { //结构错误
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            flag, err := strconv.ParseInt(data[1], 10, 64)
            if err != nil {
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            dblogic.R("Statistics").HIncrBy(okeys.GiftEventSta(sidInt64, flag), data[0], 1)
        case statistics.INVITEEVENT:
            if len(data) < 2 { //结构错误
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            flag, err := strconv.ParseInt(data[1], 10, 64)
            if err != nil {
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            dblogic.R("Statistics").HIncrBy(okeys.InviteEventSta(sidInt64, flag), data[0], 1)
        case statistics.INVITEFRIEND, statistics.INVITEFRIENDSUCCESS, statistics.COIZNER:
            if uid != "" {
                report.BuryingPoint(eventId, uid, data, ret["commonData"].(string))
            }
        case statistics.SHAREEVENT: //分享
            if len(data) < 3 || sid == "" { //参数错误
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            flag, err := strconv.ParseInt(data[0], 10, 64) //FB MESSAGE
            if err != nil {
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            point := data[1] //六个分享点
            eventIdInt64, _ := strconv.ParseInt(data[2], 10, 64)
            dblogic.R("Statistics").HIncrBy(okeys.ShareEventSta(sidInt64, flag, eventIdInt64), point, 1)
        case statistics.ACTIVITYEVENT: //活动点击
            if len(data) < 1 { //参数错误
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            dblogic.R("Statistics").HIncrBy(okeys.ActivityClickSta(), data[0], 1)
        case statistics.CALLBACKFRIEND: //好友召回 按钮点击 //1点击召回按钮数量 2点击消息数量 3成功召回数量
            if len(data) < 2 || uid == "" { //参数错误
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            flag, err := strconv.ParseInt(data[1], 10, 64) //FB MESSAGE
            if err != nil {
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            dblogic.R("Statistics").HIncrBy(okeys.FriendRecallSta(flag), uid, 1)
        case statistics.NEWVERSIONGUIDE, statistics.CREATEROLE,statistics.NEWGUIDESLOT:
            //c.JSON(http.StatusOK, getSuccessJson("ok"))
            //return
            if step == 0 {
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            dblogic.R("Statistics").HSet(okeys.Report(eventIdInt64, 0), uid, step)
            if eventId == statistics.NEWGUIDESLOT {
                step += 8
            }
            //新手步骤统计 总数
            dblogic.R("Statistics").HIncrBy(okeys.NoviceStepsSta(0), stepStr, 1)
            //新手步骤统计 FB用户 lid 1是FB 2是游客
            if ret["lid"].(string) == "1" {
                dblogic.R("Statistics").HIncrBy(okeys.NoviceStepsSta(1), stepStr, 1)
            }
        case statistics.PAYREPORT, statistics.PAYREPORTSUCCESS,statistics.PAYREPORTCANCEL,statistics.PAYFAIL:
            if step == 0 {
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            dblogic.R("Statistics").HSet(okeys.Report(eventIdInt64, 0), uid, step)
            //支付上报
            if sid != "" {
                dblogic.R("Statistics").HIncrBy(okeys.PayReport(sidInt64, eventIdInt64), stepStr, 1)
            }

            if ret["is_new_report"].(string) != "" && uid != "" {
                report.BuryingPoint(eventId, uid, data, ret["commonData"].(string))
            }
        default:
            if step == 0 {
                c.JSON(http.StatusOK, getSuccessJson("ok"))
                return
            }
            dblogic.R("Statistics").HSet(okeys.Report(eventIdInt64, 0), uid, step)
    }
    c.JSON(http.StatusOK, getSuccessJson("ok"))
    return
}
