package main

import (
	"flag"
	"fmt"
	"runtime"
	"sync"
	"time"

	"Crazy_Push/core/exp"
	"Crazy_Push/models/dblogic"
	"Crazy_Push/models/games"
	"Crazy_Push/models/log"
	"Crazy_Push/models/okeys"
	"Crazy_Push/models/ta"

	// "Crazy_Push/models/log"
	"Crazy_Push/configs"
)

var (
	lock sync.Mutex
	env  = flag.String("e", "dev", "shushu report url,value must be in [env,prod]")
)

func main() {
	flag.Parse()
	if *env != "dev" && *env != "prod" {
		fmt.Println("环境参数设置错误,可设置参数为:[dev,prod]")
		return
	}
	sysConfig.ENV = *env
	runtime.GOMAXPROCS(2) // 只跑单核
	event := []string{"user_setOnce",
		"user_set",
		"user_create",
		"new_device",
		"user_login",
		"level_up",
		"sign",
		"slot_machine",
		"attack",
		"steal",
		"building",
		"tap_fruit",
		"pet_unlock",
		"pet_feed",
		"pet_up",
		"order_finish",
		"invite_success",
		"activity_collect_reward",
		"activity_rank_reward_layer",
		"activity_rank_reward_ranking",
		"activity_coinslot",
		"activity_coinslot_quest",
		"activity_egg_knock",
		"activity_egg_rebirth",
		"activity_egg_end",
		"activity_pass_reward",
		"activity_pass_levelup",
		"chest",
		"chest_sp",
		"card_send",
		"card_accept",
		"card_ask",
		"card_respond",
		"spin_send",
		"spin_accept",
		"coin_send",
		"coin_accept",
		"cardset_complate",
		"resource_obtain",
		"resource_cost",
		"resource_list",
		"ad_cost",
		"total_recharge",
		"random_gold_card",
		"joker_card_trade",
		"spins_master",
		"guide",
		"help_repair",
		"activity_puzzle_obtain_piece",
		"activity_puzzle_reward",
		"club_card_ask",
		"club_card_send",
		"club_spin_ask",
		"club_spin_send",
		"club_create",
		"club_edit",
		"join_club",
		"quit_club",
		"club_add_friend",
		"club_add_friend_success",
		"club_expulsion",
		"club_spin_surprise",
		"club_card_surprise",
		"club_chat_surprise",
	}
	fmt.Println("请求Appid:" + sysConfig.GetTaAppId() + "请求地址:" + sysConfig.GetTaServer())
	exp.Try(func() {
		key := okeys.SSEventsList()
		eventStr, err := dblogic.R("games").SMembers(key)
		if err != nil || len(eventStr) == 0 {
			log.LogError("get_SSEventsList_err", "获取数数事件列表失败", err)
			eventStr = append(eventStr, event...)
		}
		go func() {
			ticker := time.NewTicker(5 * time.Minute)
			for {
				<-ticker.C
				eventNew, err := dblogic.R("games").SMembers(key)
				if err == nil && len(eventNew) != 0 {
					lock.Lock()
					eventStr = eventNew
					lock.Unlock()
				}
			}
			ticker.Stop()
		}()
		fmt.Printf("eventStr:%+v\n", eventStr)
		ta.SetChan(eventStr)
		taInstance := ta.NewTa()
		// taInstance.WaitGroup.Add(1)
		for _, v := range eventStr {
			go taInstance.TaDispatch(v) // 调度
			go func(eventName string) {
				for {
					taInstance.PopDataFromRedis(eventName)
					// time.Sleep(3 * time.Second) //没有数据了,休息30秒
				}
			}(v)
		}
		// taInstance.WaitGroup.Wait()
	}, func(ex exp.Exception) {
		// log.LogError("async_excepition", ex)
		PanicEvent()
	})
	for { // 阻塞进程,让程序一直跑
		time.Sleep(3 * time.Second)
	}
}

func PanicEvent() {
	en := games.NewEnWeChat()
	timeStr := time.Now().Format("2006/01/02 15:04")
	en.Send("【GO-数数统计警报】异常退出,退出时间("+timeStr+")(测试)", []string{"richard", "luokunpeng", "shaocheng"}, []uint{}, 0)
}
