package redisCli

import (
	"github.com/go-redis/redis"
	"sync"
	"time"
)

//通用redis连接
type RedisClient struct {
	Cli *redis.Client
}

var (
	redisInsts sync.Map
)

//返回单例实例
func GetRedis(host string, auth string, keys ...string) *RedisClient {
    key := host + auth
    if len(keys) > 0 {
        key = key + keys[0]
    }
	if v, ok := redisInsts.Load(key); ok {
		return v.(*RedisClient)
	}
	tmp := &RedisClient{
		Cli: redis.NewClient(&redis.Options{
			Addr:     host,
			DB:       0, // redis 数据库ID
			Password: auth,

            //PoolSize:     15, // 连接池最大socket连接数，默认为4倍CPU数， 4 * runtime.NumCPU
            //MinIdleConns: 10, //在启动阶段创建指定数量的Idle连接，并长期维持idle状态的连接数不少于指定数量；。
            ////超时
            //DialTimeout:  5 * time.Second, //连接建立超时时间，默认5秒。
            //ReadTimeout:  3 * time.Second, //读超时，默认3秒， -1表示取消读超时
            //WriteTimeout: 3 * time.Second, //写超时，默认等于读超时
            //PoolTimeout:  4 * time.Second, //当所有连接都处在繁忙状态时，客户端等待可用连接的最大等待时长，默认为读超时+1秒。
            ////闲置连接检查包括IdleTimeout，MaxConnAge
            //IdleCheckFrequency: 60 * time.Second, //闲置连接检查的周期，默认为1分钟，-1表示不做周期性检查，只在客户端获取连接时对闲置连接进行处理。
            //IdleTimeout:        5 * time.Minute,  //闲置超时，默认5分钟，-1表示取消闲置超时检查
            //MaxConnAge:         0 * time.Second,  //连接存活时长，从创建开始计时，超过指定时长则关闭连接，默认为0，即不关闭存活时长较长的连接
            ////命令执行失败时的重试策略
            //MaxRetries:      0,                      // 命令执行失败时，最多重试多少次，默认为0即不重试
            //MinRetryBackoff: 8 * time.Millisecond,   //每次计算重试间隔时间的下限，默认8毫秒，-1表示取消间隔
		}),
	}

	redisInsts.Store(key, tmp)
	v, _ := redisInsts.Load(key)
	return v.(*RedisClient)
}

//关闭出问题的连接,等待下次重新实例化
func CloseRedis(key string) {
	if _, ok := redisInsts.Load(key); !ok {
        return //key不存在
    }
    inst, _ := redisInsts.Load(key)
    inst.(*RedisClient).Cli.Close() //关闭连接
    redisInsts.Delete(key)
}

//关闭所有连接
func CloseRd() {
}

func (conn *RedisClient) Sort(key string, sort *redis.Sort) ([]string, error) {
	return conn.Cli.Sort(key, sort).Result()
}

func (conn *RedisClient) SortStore(key, store string, sort *redis.Sort) (int64, error) {
	return conn.Cli.SortStore(key, store, sort).Result()
}

func (conn *RedisClient) SortInterfaces(key string, sort *redis.Sort) ([]interface{}, error) {
	return conn.Cli.SortInterfaces(key, sort).Result()
}

func (conn *RedisClient) Touch(keys ...string) (int64, error) {
	return conn.Cli.Touch(keys...).Result()
}

func (conn *RedisClient) TTL(key string) (time.Duration, error) {
	return conn.Cli.TTL(key).Result()
}

func (conn *RedisClient) Type(key string) (string, error) {
	return conn.Cli.Type(key).Result()
}

func (conn *RedisClient) Append(key, value string) (int64, error) {
	return conn.Cli.Append(key, value).Result()
}

func (conn *RedisClient) Decr(key string) (int64, error) {
	return conn.Cli.Decr(key).Result()
}

func (conn *RedisClient) DecrBy(key string, decrement int64) (int64, error) {
	return conn.Cli.DecrBy(key, decrement).Result()
}

// Redis `GET key` command. It returns redis.Nil error when key does not exist.
func (conn *RedisClient) Get(key string) (string, error) {
	return conn.Cli.Get(key).Result()
}

func (conn *RedisClient) GetRange(key string, start, end int64) (string, error) {
	return conn.Cli.GetRange(key, start, end).Result()
}

func (conn *RedisClient) GetSet(key string, value interface{}) (string, error) {
	return conn.Cli.GetSet(key, value).Result()
}

func (conn *RedisClient) Incr(key string) (int64, error) {
	return conn.Cli.Incr(key).Result()
}

func (conn *RedisClient) IncrBy(key string, value int64) (int64, error) {
	return conn.Cli.IncrBy(key, value).Result()
}

func (conn *RedisClient) IncrByFloat(key string, value float64) (float64, error) {
	return conn.Cli.IncrByFloat(key, value).Result()
}

func (conn *RedisClient) MGet(keys ...string) ([]interface{}, error) {
	return conn.Cli.MGet(keys...).Result()
}

// MSet is like Set but accepts multiple values:
//   - MSet("key1", "value1", "key2", "value2")
//   - MSet([]string{"key1", "value1", "key2", "value2"})
//   - MSet(map[string]interface{}{"key1": "value1", "key2": "value2"})
func (conn *RedisClient) MSet(values ...interface{}) (string, error) {
	return conn.Cli.MSet(values...).Result()
}

// MSetNX is like SetNX but accepts multiple values:
//   - MSetNX("key1", "value1", "key2", "value2")
//   - MSetNX([]string{"key1", "value1", "key2", "value2"})
//   - MSetNX(map[string]interface{}{"key1": "value1", "key2": "value2"})
func (conn *RedisClient) MSetNX(values ...interface{}) (bool, error) {
	return conn.Cli.MSetNX(values...).Result()
}

// Redis `SET key value [expiration]` command.
// Use expiration for `SETEX`-like behavior.
//
// Zero expiration means the key has no expiration time.
// KeepTTL(-1) expiration is a Redis KEEPTTL option to keep existing TTL.
func (conn *RedisClient) Set(key string, value interface{}, expiration time.Duration) (string, error) {
	return conn.Cli.Set(key, value, expiration).Result()
}

// Redis `SET key value [expiration] NX` command.
//
// Zero expiration means the key has no expiration time.
// KeepTTL(-1) expiration is a Redis KEEPTTL option to keep existing TTL.
func (conn *RedisClient) SetNX(key string, value interface{}, expiration time.Duration) (bool, error) {
	return conn.Cli.SetNX(key, value, expiration).Result()
}

//// Redis `SETEX key expiration value` command.
//func (conn *RedisClient) SetEx(key string, value interface{}, expiration time.Duration) (bool, error) {
//	return conn.Cli.SetEx(key, value, expiration).Result()
//}

// Redis `SET key value [expiration] XX` command.
//
// Zero expiration means the key has no expiration time.
// KeepTTL(-1) expiration is a Redis KEEPTTL option to keep existing TTL.
func (conn *RedisClient) SetXX(key string, value interface{}, expiration time.Duration) (bool, error) {
	return conn.Cli.SetXX(key, value, expiration).Result()
}

func (conn *RedisClient) SetRange(key string, offset int64, value string) (int64, error) {
	return conn.Cli.SetRange(key, offset, value).Result()
}

func (conn *RedisClient) StrLen(key string) (int64, error) {
	return conn.Cli.StrLen(key).Result()
}

//------------------------------------------------------------------------------

func (conn *RedisClient) HDel(key string, fields ...string) (int64, error) {
	return conn.Cli.HDel(key, fields...).Result()
}

func (conn *RedisClient) HExists(key, field string) (bool, error) {
	return conn.Cli.HExists(key, field).Result()
}

func (conn *RedisClient) HGet(key, field string) (string, error) {
	return conn.Cli.HGet(key, field).Result()
}

func (conn *RedisClient) HGetAll(key string) (map[string]string, error) {
	return conn.Cli.HGetAll(key).Result()
}

func (conn *RedisClient) HIncrBy(key, field string, incr int64) (int64, error) {
	return conn.Cli.HIncrBy(key, field, incr).Result()
}

func (conn *RedisClient) HIncrByFloat(key, field string, incr float64) (float64, error) {
	return conn.Cli.HIncrByFloat(key, field, incr).Result()
}

func (conn *RedisClient) HKeys(key string) ([]string, error) {
	return conn.Cli.HKeys(key).Result()
}

func (conn *RedisClient) HLen(key string) (int64, error) {
	return conn.Cli.HLen(key).Result()
}

func (conn *RedisClient) HMGet(key string, fields ...string) ([]interface{}, error) {
	return conn.Cli.HMGet(key, fields...).Result()
}

func (conn *RedisClient) HSet(key string, field string, value interface{}) (bool, error) {
	return conn.Cli.HSet(key, field, value).Result()
}

// HMSet is a deprecated version of HSet left for compatibility with Redis 3.
func (conn *RedisClient) HMSet(key string, fields map[string]interface{}) (string, error) {
	return conn.Cli.HMSet(key, fields).Result()
}

func (conn *RedisClient) HSetNX(key, field string, value interface{}) (bool, error) {
	return conn.Cli.HSetNX(key, field, value).Result()
}

func (conn *RedisClient) HVals(key string) ([]string, error) {
	return conn.Cli.HVals(key).Result()
}

//------------------------------------------------------------------------------

func (conn *RedisClient) BLPop(timeout time.Duration, keys ...string) ([]string, error) {
	return conn.Cli.BLPop(timeout, keys...).Result()
}

func (conn *RedisClient) BRPop(timeout time.Duration, keys ...string) ([]string, error) {
	return conn.Cli.BRPop(timeout, keys...).Result()
}

func (conn *RedisClient) BRPopLPush(source, destination string, timeout time.Duration) (string, error) {
	return conn.Cli.BRPopLPush(source, destination, timeout).Result()
}

func (conn *RedisClient) LIndex(key string, index int64) (string, error) {
	return conn.Cli.LIndex(key, index).Result()
}

func (conn *RedisClient) LInsert(key, op string, pivot, value interface{}) (int64, error) {
	return conn.Cli.LInsert(key, op, pivot, value).Result()
}

func (conn *RedisClient) LInsertBefore(key string, pivot, value interface{}) (int64, error) {
	return conn.Cli.LInsertBefore(key, pivot, value).Result()
}

func (conn *RedisClient) LInsertAfter(key string, pivot, value interface{}) (int64, error) {
	return conn.Cli.LInsertAfter(key, pivot, value).Result()
}

func (conn *RedisClient) LLen(key string) (int64, error) {
	return conn.Cli.LLen(key).Result()
}

func (conn *RedisClient) LPop(key string) (string, error) {
	return conn.Cli.LPop(key).Result()
}

func (conn *RedisClient) LPush(key string, values ...interface{}) (int64, error) {
	return conn.Cli.LPush(key, values...).Result()
}

func (conn *RedisClient) LRange(key string, start, stop int64) ([]string, error) {
	return conn.Cli.LRange(key, start, stop).Result()
}

func (conn *RedisClient) LRem(key string, count int64, value interface{}) (int64, error) {
	return conn.Cli.LRem(key, count, value).Result()
}

func (conn *RedisClient) LSet(key string, index int64, value interface{}) (string, error) {
	return conn.Cli.LSet(key, index, value).Result()
}

func (conn *RedisClient) LTrim(key string, start, stop int64) (string, error) {
	return conn.Cli.LTrim(key, start, stop).Result()
}

func (conn *RedisClient) RPop(key string) (string, error) {
	return conn.Cli.RPop(key).Result()
}

func (conn *RedisClient) RPopLPush(source, destination string) (string, error) {
	return conn.Cli.RPopLPush(source, destination).Result()
}

func (conn *RedisClient) RPush(key string, values ...interface{}) (int64, error) {
	return conn.Cli.RPush(key, values...).Result()
}

//------------------------------------------------------------------------------

func (conn *RedisClient) SAdd(key string, members ...interface{}) (int64, error) {
	return conn.Cli.SAdd(key, members...).Result()
}

func (conn *RedisClient) SCard(key string) (int64, error) {
	return conn.Cli.SAdd(key).Result()
}

func (conn *RedisClient) SDiff(keys ...string) ([]string, error) {
	return conn.Cli.SDiff(keys...).Result()
}

func (conn *RedisClient) SIsMember(key string, member interface{}) (bool, error) {
	return conn.Cli.SIsMember(key, member).Result()
}

// Redis `SMEMBERS key` command output as a slice.
func (conn *RedisClient) SMembers(key string) ([]string, error) {
	return conn.Cli.SMembers(key).Result()
}

func (conn *RedisClient) SMove(source, destination string, member interface{}) (bool, error) {
	return conn.Cli.SMove(source, destination, member).Result()
}

// Redis `SPOP key` command.
func (conn *RedisClient) SPop(key string) (string, error) {
	return conn.Cli.SPop(key).Result()
}

// Redis `SRANDMEMBER key` command.
func (conn *RedisClient) SRandMember(key string) (string, error) {
	return conn.Cli.SRandMember(key).Result()
}

func (conn *RedisClient) SRem(key string, members ...interface{}) (int64, error) {
	return conn.Cli.SRem(key, members...).Result()
}

func (conn *RedisClient) SUnion(keys ...string) ([]string, error) {
	return conn.Cli.SUnion(keys...).Result()
}

func (conn *RedisClient) SUnionStore(destination string, keys ...string) (int64, error) {
	return conn.Cli.SUnionStore(destination, keys...).Result()
}

// Redis `ZADD key score member [score member ...]` command.
func (conn *RedisClient) ZAdd(key string, members ...redis.Z) (int64, error) {
	return conn.Cli.ZAdd(key, members...).Result()
}

// Redis `ZADD key INCR score member` command.
func (conn *RedisClient) ZIncr(key string, member redis.Z) (float64, error) {
	return conn.Cli.ZIncr(key, member).Result()
}

func (conn *RedisClient) ZCard(key string) (int64, error) {
	return conn.Cli.ZCard(key).Result()
}

func (conn *RedisClient) ZCount(key, min, max string) (int64, error) {
	return conn.Cli.ZCount(key, min, max).Result()
}

func (conn *RedisClient) ZIncrBy(key string, increment float64, member string) (float64, error) {
	return conn.Cli.ZIncrBy(key, increment, member).Result()
}

func (conn *RedisClient) ZInterStore(destination string, store redis.ZStore) (int64, error) {
	return conn.Cli.ZInterStore(destination, store).Result()
}

func (conn *RedisClient) ZRange(key string, start, stop int64) ([]string, error) {
	return conn.Cli.ZRange(key, start, stop).Result()
}

func (conn *RedisClient) ZRangeWithScores(key string, start, stop int64) ([]redis.Z, error) {
	return conn.Cli.ZRangeWithScores(key, start, stop).Result()
}

func (conn *RedisClient) ZRangeByScore(key string, opt redis.ZRangeBy) ([]string, error) {
	return conn.Cli.ZRangeByScore(key, opt).Result()
}

func (conn *RedisClient) ZRank(key, member string) (int64, error) {
	return conn.Cli.ZRank(key, member).Result()
}

func (conn *RedisClient) ZRem(key string, members ...interface{}) (int64, error) {
	return conn.Cli.ZRem(key, members...).Result()
}

func (conn *RedisClient) ZRemRangeByRank(key string, start, stop int64) (int64, error) {
	return conn.Cli.ZRemRangeByRank(key, start, stop).Result()
}

func (conn *RedisClient) ZRemRangeByScore(key, min, max string) (int64, error) {
	return conn.Cli.ZRemRangeByScore(key, min, max).Result()
}

func (conn *RedisClient) ZRevRange(key string, start, stop int64) ([]string, error) {
	return conn.Cli.ZRevRange(key, start, stop).Result()
}

func (conn *RedisClient) ZRevRangeWithScores(key string, start, stop int64) ([]redis.Z, error) {
	return conn.Cli.ZRevRangeWithScores(key, start, stop).Result()
}

func (conn *RedisClient) ZRevRangeByScore(key string, opt redis.ZRangeBy) ([]string, error) {
	return conn.Cli.ZRevRangeByScore(key, opt).Result()
}

func (conn *RedisClient) ZRevRank(key, member string) (int64, error) {
	return conn.Cli.ZRevRank(key, member).Result()
}

func (conn *RedisClient) ZScore(key, member string) (float64, error) {
	return conn.Cli.ZScore(key, member).Result()
}

func (conn *RedisClient) ZUnionStore(dest string, store redis.ZStore) (int64, error) {
	return conn.Cli.ZUnionStore(dest, store).Result()
}

func (conn *RedisClient) Expire(key string, expiration time.Duration) (bool, error) {
	return conn.Cli.Expire(key, expiration).Result()
}

func (conn *RedisClient) ExpireAt(key string, tm time.Time) (bool, error) {
	return conn.Cli.ExpireAt(key, tm).Result()
}
