package ab

import (
	"context"
	"encoding/json"
	"fmt"
	"net/http"
	"strings"
	"sync"

	"Crazy_Push/models/dblogic"
	"Crazy_Push/models/okeys"
	"Crazy_Push/models/utils"
	tkLog "Crazy_Push/models/utils/log"
	"Crazy_Push/models/utils/tracing"
)

var (
	authKeys = [3]string{
		"AAAAwSTdJJQ:APA91bGHYd9LFuSfFZ7gPm5lbAvdb-1xoW4h4l1vbRMcn3aB7u8OhIXIWgLw58xZpuSINmmb_4v0ZkZZ2cO7lwKMu_kDuliTP8I1PovhSD4LSiUv4toAqnHFt60pGGbJ30z1-3Vi3a1X", //crazy spin
		"AAAAUHHAsMY:APA91bFYCxoaTb0J0XW2hNsWD6gLVn0UwXncw-gr4Ww8_Mh1LJbarOAhRWGR14WAEWE_MyCfOonUnE1bl-qA0nOTux_b2L-n00h-Vg_tpZGx-bBl85FG6g5LnOUtxba9pNc8Tse_Igje", //crazy coin
		"AAAAwSTdJJQ:APA91bGHYd9LFuSfFZ7gPm5lbAvdb-1xoW4h4l1vbRMcn3aB7u8OhIXIWgLw58xZpuSINmmb_4v0ZkZZ2cO7lwKMu_kDuliTP8I1PovhSD4LSiUv4toAqnHFt60pGGbJ30z1-3Vi3a1X", //crazyfox
	}
	//firebaesHeader = make(http.Header)
	chanhttpClient = make(chan *httpClient, 50) // Channel for new join users.
	pushUrl        = "https://fcm.googleapis.com/fcm/send"
)

type RdsStr struct {
	Token   string      `json:"token"`
	Title   string      `json:"title"`
	Content string      `json:"content"`
	Sid     string      `json:"sid"`
	Data    interface{} `json:"data"`
	Img     string      `json:"img"`
	Label   string      `json:"label"`
}

type RspMsg struct {
	MulticastId  float64             `json:"multicast_id"`
	Success      int                 `json:"success"`
	Failure      int                 `json:"failure"`
	CanonicalIds int                 `json:"canonical_ids"`
	Results      []map[string]string `json:"results"`
}

type PushMsg struct {
	To           string            `json:"to"`
	Priority     string            `json:"priority"`
	Notification map[string]string `json:"notification"`
	//Data string `json:"data,omitempty"` //忽略空
	Data map[string]string `json:"data,omitempty"` //忽略空
}

type firebase struct {
	Header    [3]http.Header
	WaitGroup *sync.WaitGroup
}

func NewFirebase() *firebase {
	header := make(http.Header)
	header.Set("Content-type", "application/json")
	header.Set("Authorization", "key="+authKeys[0])

	header2 := make(http.Header)
	header2.Set("Content-type", "application/json")
	header2.Set("Authorization", "key="+authKeys[1])

	header3 := make(http.Header)
	header3.Set("Content-type", "application/json")
	header3.Set("Authorization", "key="+authKeys[2])
	return &firebase{
		Header: [3]http.Header{
			header, header2, header3,
		},
		WaitGroup: &sync.WaitGroup{},
	}
}

//json encode
func (base *firebase) ParsePushEncode(params *PushMsg) string {
	str, err := json.Marshal(params)
	if err != nil {
		return ""
	}
	return string(str)
}

func (base *firebase) NewPushEvent(str string) {
	rds := &RdsStr{}
	err := json.Unmarshal([]byte(str), &rds)
	if err != nil {
		tkLog.TKLogError(context.Background(), 0, "unmarshal redis json data error:"+err.Error())
		//fmt.Println(fmt.Sprintf("unmarshal redis json data error:%s", str))
		return
	}

	notification := map[string]string{
		"title": rds.Title,
		"body":  rds.Content,
	}

	if rds.Sid == "2" { //ios设置角标
		notification["badge"] = "1"
	}

	params := &PushMsg{
		To:           rds.Token,
		Priority:     "high",
		Notification: notification,
	}
	if rds.Img != "" {
		params.Data = map[string]string{
			"img":   rds.Img,
			"label": rds.Label,
		}
	}

	paramsStr := base.ParsePushEncode(params)
	chanhttpClient <- &httpClient{Params: paramsStr, Url: pushUrl, Header: base.Header[0], Key: str}
	return
}

type FCMSendResponse struct {
	MulticastID  int64 `json:"multicast_id"`
	Success      int   `json:"success"`
	Failure      int   `json:"failure"`
	CanonicalIds int   `json:"canonical_ids"`
	Results      []struct {
		Error     string `json:"error"`
		MessageID string `json:"message_id"`
	} `json:"results"`
}

//发送请求并等待响应
func (base *firebase) ReqAndRspDeal(client *httpClient) {
	defer base.WaitGroup.Done()
	////推送给crazyfox
	ctx := tracing.ContextWithTrace("PushService")
	//base.PushToCrazyfox(client.Params, base.Header[2], client.Key)

	res, code := client.PushRequest()
	if code == http.StatusOK {
		var rsp FCMSendResponse
		if err := json.Unmarshal([]byte(res), &rsp); err != nil {
			tkLog.TKLogError(ctx, 0, fmt.Sprintf("Unmarshal FCMSendResponse failed,err=%+v", err))
		}
		switch {
		case rsp.Success == 1:
			tkLog.TkLogInfo(ctx, 0, fmt.Sprintf("FCM推送成功:%+v", client.Params))
			client = nil
		case rsp.Failure == 1:
			for _, result := range rsp.Results {
				if !strings.Contains(result.Error, "NotRegistered") {
					tkLog.TKLogError(ctx, 0, fmt.Sprintf("FCM推送失败,err=%+v, param=%+v", result.Error, client.Params))
				}
			}
		}
	} else {
		tkLog.TKLogError(ctx, 0, fmt.Sprintf("fcm/send失败,code=%d,resp=%+v,param=%+v", code, res, client.Params))
	}
	////推送失败,推送第二个authkey
	//if client.Header.Get("Authorization") == "key="+authKeys[0] {
	//	chanhttpClient <- &httpClient{Params: client.Params, Url: pushUrl, Header: base.Header[1], Key: client.Key}
	//}

	////推送失败,推送第三个authkey
	//if client.Header.Get("Authorization") == "key="+authKeys[0] {
	//	chanhttpClient <- &httpClient{Params: client.Params, Url: pushUrl, Header: base.Header[2], Key: client.Key}
	//}
	//client = nil
	return
}

func (base *firebase) PushToCrazyfox(param string, header http.Header, key string) {

	err := utils.GoroutinePool.Submit(func() {
		utils.Try(func() {
			clientCrazyfox := &httpClient{
				Url:    pushUrl,
				Params: param,
				Header: header,
				Key:    key,
			}
			res, code := clientCrazyfox.PushRequest()
			if code == 200 && strings.Contains(res, `"success":1`) {
				tkLog.TkLogInfo(context.Background(), 0, "crazyfox 推送给成功"+res+"\n"+param)
			} else {
				tkLog.TKLogError(context.Background(), 0, "crazyfox 推送失败"+res+"\n"+param)
			}
		}, func(ex utils.Exception) {
			tkLog.TKLogError(context.Background(), 0, "crazyfox 推送失败远程请求返回异常"+ex.String())
		})
	})
	if err != nil {
		//Todo:elk
		tkLog.TKLogError(context.Background(), 0, "crazyfox 协程运行失败"+err.Error())

	}
}

//调度
func (base *firebase) HttpDispatch() {
	for {
		select {
		case client := <-chanhttpClient:
			base.WaitGroup.Add(1)
			go base.ReqAndRspDeal(client)
		case <-interrupt:
			close(chanhttpClient)
			close(interrupt)
			base.WaitGroup.Wait()
			return //退出
		}
	}
}

func (base *firebase) PopDataFromRedis() (string, error) {
	return dblogic.R("Games").RPop(okeys.CommonKeys("FireBaseList"))
	//return dblogic.R().LRange(okeys.CommonKeys("FireBaseList"), 0, 100)
	//return dblogic.R().LRange(okeys.CommonKeys("FireBaseList"), 0, 1)
}
