package ab

import (
    "fmt"
    "net/http"
    "encoding/json"
    "sync"
    "strings"

    "Crazy_Push/models/dblogic"
    "Crazy_Push/models/okeys"
    "Crazy_Push/models/log"
)

var (
	chanhttpClientNew = make(chan *httpClient, 50) // Channel for new join users.
    pushUrlNew = "https://fcm.googleapis.com/v1/projects/crazycoin-1dc18/messages:send"
)

type RdsStrNew struct {
    Token string `json:"token"`
    Title string `json:"title"`
    Content string `json:"content"`
    Sid string `json:"sid"`
    Data interface{} `json:"data"`
    Img string `json:"img"`
    Label string `json:"label"`
}

type RspMsgNew struct {
    MulticastId float64 `json:"multicast_id"`
    Success int `json:"success"`
    Failure int `json:"failure"`
    CanonicalIds int `json:"canonical_ids"`
    Results []map[string]string `json:"results"`
}

type PushMsgNew struct {
    Topic string `json:"topic"`
    //To string `json:"to"`
    //Priority string `json:"priority"`
    Notification map[string]string `json:"notification"`
    //Data string `json:"data,omitempty"` //忽略空
    Data map[string]string `json:"data,omitempty"` //忽略空
    //FcmOptions map[string]string `json:"fcm_options"`
    Android map[string]map[string]string `json:"android"`
    Apns  map[string]map[string]string `json:"apns"`
}

type firebaseNew struct {
    Header http.Header
    WaitGroup *sync.WaitGroup
}

func NewFirebaseNew() *firebaseNew {
    header := make(http.Header)
    header.Set("Content-type", "application/json")
    return &firebaseNew{
        Header: header,
        WaitGroup: &sync.WaitGroup{},
    }
}

//json encode
func (base2 *firebaseNew) ParsePushEncode(params map[string]*PushMsgNew) string {
    str, err := json.Marshal(params)
    if err != nil {
        return ""
    }
    return string(str)
}

func (base2 *firebaseNew) NewPushEvent(str string) {
    rds := &RdsStrNew{}
    err := json.Unmarshal([]byte(str), &rds)
    if err != nil {
        fmt.Println("unmarshal redis json data error", err)
        return
    }

    anysLabel := "default"
    if rds.Label != "" {
        anysLabel = rds.Label
    }

    notification := map[string]string{
        "title": rds.Title,
        "body": rds.Content,
    }
    if rds.Sid == "2" { //ios设置角标
        notification["badge"] = "1"
    }

    fcmOptions := map[string]string{"analytics_label": anysLabel}
    params := &PushMsgNew {
        Topic: anysLabel,
        Notification: notification,
        //FcmOptions: fcmOptions,
    }
    if rds.Sid == "2" {
        params.Apns = make(map[string]map[string]string)
        params.Apns["fcm_options"] = fcmOptions
    } else {
        params.Android = make(map[string]map[string]string)
        params.Android["fcm_options"] = fcmOptions
    }

    if(rds.Img != ""){
        params.Data = map[string]string{
            "img": rds.Img,
            "label": rds.Label,
        }
    }

    base2.Header.Set("Content-type", "application/json")
    base2.Header.Set("Authorization", "Bearer " + rds.Token)
    paramsStr := base2.ParsePushEncode(map[string]*PushMsgNew{"message": params})
    chanhttpClientNew <- &httpClient{Params: paramsStr, Url: pushUrlNew, Header: base2.Header, Key: str}
    return
}

//发送请求并等待响应
func (base2 *firebaseNew) ReqAndRspDeal(client *httpClient) {
    defer base2.WaitGroup.Done()
    res, code := client.PushRequest()
    if code == 200 && strings.Contains(res, `"success":1`){
        fmt.Println("推送成功", res)
        client = nil
        return
    } else {
        log.LogInfo("firebase_errr", "推送失败远程请求返回异常", res)
    }
    ////推送失败,推送第二个authkey
    //if client.Header.Get("Authorization") == "key="+authKeys[1] {
    //    chanhttpClientNew <- &httpClient{Params: client.Params, Url: pushUrlNew, Header: base2.Header[1], Key: client.Key}
    //}
    client = nil

    //res, code := client.PushRequest()
    //if code != 200 {
    //    fmt.Println("push firebaseNew请求异常")
    //    return
    //}
    //msg := &RspMsgNew{}
    //err := json.Unmarshal([]byte(res), &msg)
    //if err != nil {
    //    fmt.Println("push firebaseNew返回json结构错误", res)
    //    return
    //}
    //if msg.Success == 1 {
    //    fmt.Println("推送成功")
    //    //删除已推送过的消息key
    //    //dblogic.R().LRem(okeys.CommonKeys("FireBaseList"), 0, client.Key)
    //} else {
    //    //重新推一次
    //    chanhttpClientNew <- &httpClient{Params: client.Params, Url: pushUrlNew, Header: base2.Header[1], Key: client.Key}
    //    //dblogic.R().LRem(okeys.CommonKeys("FireBaseList"), 0, client.Key)
    //}
    return
}

//调度
func (base2 *firebaseNew) HttpDispatch() {
	for {
		select {
		case client := <-chanhttpClientNew:
            base2.WaitGroup.Add(1)
            go base2.ReqAndRspDeal(client)
		case <-interrupt:
            close(chanhttpClientNew)
            close(interrupt)
            base2.WaitGroup.Wait()
            return //退出
        //default:
        //    time.Sleep(3 * time.Millisecond) //3毫秒推一次,频率太高http请求无响应
		}
	}
}

func (base2 *firebaseNew) PopDataFromRedis() (string, error) {
    return dblogic.R("Games").RPop(okeys.CommonKeys("FireBaseList"))
}
