package ab

import (
	"flag"
	"log"
	"net/url"
	"os"
	"os/signal"
	"encoding/json"
	"time"
	"sync"
    "fmt"
    "strconv"

    "Crazy_Push/models/games"
    "Crazy_Push/models/okeys"
    "Crazy_Push/models/dblogic"
    "Crazy_Push/core/function"
    logger "Crazy_Push/models/log"

	"github.com/gorilla/websocket"
)

//const SOCKETSERVER = "testapi.superant.cc:8282"
//const SOCKETSERVER = "120.78.8.90:8282"
//const SOCKETSERVER = "47.115.186.236:8282"
//const SOCKETSERVER = "127.0.0.1:8089"
//const SOCKETSERVER = "8.135.54.210:8282"
//const SOCKETSERVER = "8.135.40.225:8282"
const SOCKETSERVER = "10.0.5.150:8282"


var (
    //addr          = flag.String("addr", "testworker.superant.cc:80", "http service address")
    //addr          = flag.String("addr", "47.253.12.223", "http service address")
    //addr          = flag.String("addr", "127.0.0.1:8089", "http service address")
    addr          = flag.String("addr", SOCKETSERVER, "http service address")
    //addr          = flag.String("addr", "172.17.0.4:8282", "http service address")

	chanwsClient  = make(chan *wsClient, 50) // Channel for new join users.
    chanSend      = make(chan map[string]string, 50)
	interrupt     = make(chan os.Signal, 1)

    //Clients       = list.New() //ws连接用户链表
    Clients       sync.Map
    lock          sync.RWMutex
    total         = 0
)

type wsClient struct {
    Uid string
    Conn *websocket.Conn
}

func Join(uid string) {
    chanwsClient <- InitClient(uid)
}

func InitClient(uid string) *wsClient {
    //u := url.URL{Scheme: "ws", Host: *addr, Path: ""}

    query := make(url.Values)
    query.Add("uid", uid)
    query.Add("token", "aaaa1")

    u := url.URL{Scheme: "ws", Host: *addr, RawQuery: query.Encode()}

    //设置超时时间
    websocket.DefaultDialer.HandshakeTimeout = 300 * time.Second
	c, _, err := websocket.DefaultDialer.Dial(u.String(), nil)
	if err != nil {
		log.Fatal("dial:", err)
        return nil
	}
    return &wsClient{Uid:uid, Conn:c}
}

//统一发送消息
func Send(uid string, msg string) {
    m := map[string]string{"uid": uid, "msg": msg}
    chanSend<-m
}

func SendHeartbeat(uid string) {
    cmd := map[string]string{"cmd": "8200"}
    for {
        time.Sleep(3 * time.Second)
        msg, _ := json.Marshal(cmd)
        Send(uid, string(msg))
    }
}

func SendSlot(uid string, sleep int) {
    //strCmd := dblogic.R().HGet("USERBUILDLEVEL:"+uid, "levelId")
    levelId := "7"
    //if strCmd.String() != "" {
    //    levelId = strCmd.String()
    //}
    cmd := map[string]string{"cmd":"12291","uid":uid,"levelId":levelId,"maxBet":"1","multiple":"1","autoSpins":"1", "nnn":"5"}
    for {
        time.Sleep(time.Duration(sleep) * time.Second)

        intUid, _ := strconv.ParseInt(uid, 10, 64)
        dblogic.R("Games").HSet(okeys.UserAttr(intUid), "money", 250000)
        dblogic.R("Games").HSet(okeys.UserAttr(intUid), "spins", 30)

        msg, _ := json.Marshal(cmd)
        Send(uid, string(msg))
    }
}

func (ws *wsClient) SendMsg(msg string) bool {
    lock.Lock()
    err := ws.Conn.WriteMessage(websocket.TextMessage, []byte(msg))
    lock.Unlock()
    log.Println("send msg")
    log.Println(msg)
    if err != nil {
        log.Println("write error:", err)
        Join(ws.Uid)
        logger.LogError(SOCKETSERVER, fmt.Sprintf("\n当前websocket断开", function.Date("2006-01-02 15:04:05", time.Now().Unix())))
        //ws.Conn.Close()
        return false
    }
    return true
}

//调度
func WsDispatch() {
	for {
		select {
		case ws, opened := <-chanwsClient:
            if !opened {
                break
            }
            if _, ok := Clients.Load(ws.Uid); ok {
				log.Println("Old user:", ws.Uid, ";WebSocket:", ws.Conn != nil)
            } else {
				Clients.Store(ws.Uid, ws) // Add user to the end of list.
                log.Println("New user:", ws.Uid, ";WebSocket:", ws.Conn != nil)
                lock.Lock()
                total++
                lock.Unlock()
            }
		case m, opened := <-chanSend:
            if !opened {
                break
            }
            if v, ok := Clients.Load(m["uid"]); ok {
                v.(*wsClient).SendMsg(m["msg"]) //发送消息
            }
		case <-interrupt:
            Clients.Range(func(k, v interface{}) bool {
                err := v.(*wsClient).Conn.WriteMessage(websocket.CloseMessage, websocket.FormatCloseMessage(websocket.CloseNormalClosure, ""))
                v.(*wsClient).Conn.Close()
                Clients.Delete(k)
                log.Println("删除ws连接:", k)
                if err != nil {
                    log.Println("write close:", err)
                    return false
                }
                return true
            })
            log.Println("手动关闭程序")
            os.Exit(1)
		}
	}
}

func init() {
	flag.Parse()
	log.SetFlags(0)
}

func Loop() {
	for {
        Clients.Range(func(k, ws interface{}) bool {
            if ws.(*wsClient).Conn == nil {
                logger.LogError(SOCKETSERVER, fmt.Sprintf("\n当前websocket断开", function.Date("2006-01-02 15:04:05", time.Now().Unix())))
                return true
            }
            _, p, err := ws.(*wsClient).Conn.ReadMessage()
            if err != nil && ws != nil {
                //return
                log.Println("read msg err by uid:", ws.(*wsClient).Uid, p, err)
                logger.LogError(SOCKETSERVER, fmt.Sprintf("\n当前websocket断开", function.Date("2006-01-02 15:04:05", time.Now().Unix())))
                //ws.(*wsClient).Conn.WriteMessage(websocket.CloseMessage, websocket.FormatCloseMessage(websocket.CloseNormalClosure, ""))
                //lock.Lock()
                //mlog.LogInst().LogInfo(fmt.Sprintf("当前用户量:%d\n", total))
                //lock.Unlock()
                ws.(*wsClient).Conn.Close()
                Clients.Delete(k)
                //Join(ws.(*wsClient).Uid)
            } else {
                if len(string(p)) > 30 {
                    fmt.Println("收到消息1", games.MsgDecode(string(p)))
                } else{
                    fmt.Println("收到消息2", string(p))
                }
                log.Println("received msg by uid:", ws.(*wsClient).Uid, string(p))
                logger.LogInfo(SOCKETSERVER, fmt.Sprintf("\n连接正常:%s", function.Date("2006-01-02 15:04:05", time.Now().Unix())))
            }
            return true
        })
    }
}

func InitWs(uid string, sleep int, url ...string) {
    if len(url) > 0 {
        addr = flag.String("addr1", url[0], "http service address")
    }
    //添加一个客户连接
    Join(uid)
    //cmd := map[string]string{"cmd": "0x2008"}
    //cmd := map[string]string{"cmd": "8200"}
    //msg, _ := json.Marshal(cmd)
    //Send(uid, string(msg))
    go WsDispatch()
    //for {
    //    Send(uid, string(msg))
    //    time.Sleep(100 * time.Millisecond)
    //    //time.Sleep(100 * time.Second)
    //}
    ////cmd = map[string]string{"cmd": "0x1001", "uid": "262593"}
    intUid, _ := strconv.ParseInt(uid, 10, 64)
    token, _ := dblogic.R("Games").Get(okeys.UserToken(intUid))
    cmd := map[string]string{"cmd": "4097", "uid": uid, "token": token}
    msg, _ := json.Marshal(cmd)
    Send(uid, string(msg))

    //cmd = map[string]string{"cmd": "28673", "uid": uid, "lid": "1", "sid": "1", "mod": "Diamond", "act": "getDiamondGoods", "service": "1"}
    //msg, _ = json.Marshal(cmd)
    //Send(uid, string(msg))

    go SendHeartbeat(uid)
    go SendSlot(uid, sleep)

	signal.Notify(interrupt, os.Interrupt)
    //go WsDispatch()
}
