package games

import (
    "time"
    //"encoding/json"
    "fmt"
    "strconv"

    "Crazy_Push/models/log"
    "Crazy_Push/models/dblogic"
    "Crazy_Push/models/tables"
    "Crazy_Push/models/okeys"

    "github.com/go-redis/redis"
)

//成就相关
type Achievement struct {
}

//成就数据表和redis结构不一致处理
type TAchievementRedis struct {
    *tables.TAchievement
    IsLanding int64 `json:"isLanding"`
}

func NewAchievement() *Achievement {
    return &Achievement{}
}

//获取玩家成就
func (ach *Achievement) GetUserAchieve(user *UserInfo) []*TAchievementRedis {
    key := okeys.UserAchieve(user.Uid)
    tmp, err := dblogic.R("Games").HGetAll(key)

    if err != nil {
        log.LogError("achievement_err", "redis获取玩家成就信息失败", user.Uid, err)
    }

    var tb []*TAchievementRedis
    achievement := &tables.TAchievement{}
    if len(tmp) == 0 { //没有取到数据，从数据表中获取
        tname := achievement.TableName()+strconv.FormatInt(user.Uid % 100, 10)
        dblogic.M(dblogic.Achievement).
            Table(tname).
            Where("uid = ?", user.Uid).Find(&tb)

        if len(tb) == 0 {
            fmt.Println("未找到玩家"+ strconv.FormatInt(user.Uid, 10)+"的成就数据")
            log.LogError("achievement_err", "未找到玩家的成就数据", user.Uid)
            return tb
        }

        for _, v := range tb {
            resStr, _ := json.Marshal(v)
            dblogic.R("Games").HSetNX(key, strconv.FormatInt(v.Aid, 10), resStr)
        }
        dblogic.R("Games").Expire(key, time.Second * 6 * 3600)
    }else{
        for _, v := range tmp {
            tmpAchievement := &TAchievementRedis{}
            err := json.Unmarshal([]byte(v), tmpAchievement)
            if err != nil {
                fmt.Println("json解析玩家"+strconv.FormatInt(user.Uid, 10)+"的成就数据")
                log.LogError("achievement_err", "json解析玩家成就数据失败", user.Uid, err)
                return tb
            }
            tb = append(tb, tmpAchievement)
        }
    }
    return tb
}

//根据aid获取玩家成就信息
func (ach *Achievement) GetUserAchieveByAid(user *UserInfo, aid int64) *TAchievementRedis {
    key := okeys.UserAchieve(user.Uid)
    tmp, err := dblogic.R("Games").HGet(key, strconv.FormatInt(aid, 10))
    if err != nil {
        log.LogError("achievement_err", "redis通过aid获取玩家成就信息失败", user.Uid, aid, err)
    }
    data := &TAchievementRedis{}
    if tmp != "" {
        err := json.Unmarshal([]byte(tmp), data)
        if err != nil {
            fmt.Println("json解析玩家"+strconv.FormatInt(user.Uid, 10)+"的成就aid数据失败", aid)
            log.LogError("achievement_err", "json解析玩家成就aid数据失败", user.Uid, aid, err)
        }
        return data
    }
    arr := ach.GetUserAchieve(user)
    if len(arr) == 0 {
        return ach.InsertUserAchieve(user, aid)
    }
    for _, v := range arr {
        if v.Aid == aid {
            return v
        }
    }
    return data
}

//插入成就信息
func (ach *Achievement) InsertUserAchieve(user *UserInfo, aid int64) *TAchievementRedis {
    tb := &tables.TAchievement{
        Uid: user.Uid,
        Aid: aid,
        Process: 0,
        Stage: 0,
    }
    retStr, _ := json.Marshal(&tb)
    _, err := dblogic.R("Games").HSetNX(okeys.UserAchieve(user.Uid), strconv.FormatInt(aid, 10), string(retStr))
    //并发锁
    if err != nil {
        log.LogError("achievement_err", "写入玩家成就aid数据错误", user.Uid, tb)
    }
    tname := tb.TableName()+strconv.FormatInt(user.Uid % 100, 10)
    dblogic.M(dblogic.Achievement).Table(tname).Create(tb)
    return &TAchievementRedis{tb, 0}
}

//更新玩家成就信息
func (ach *Achievement) UpdateUserAchieve(user *UserInfo, aid int64, ret *TAchievementRedis) {
    key := okeys.UserAchieve(user.Uid)
    ret.IsLanding = 1
    retStr, err := json.Marshal(ret)
    if err != nil {
        log.LogError("achievement_err", "json编码玩家成就aid数据失败", user.Uid, aid, err)
        return
    }
    dblogic.R("Games").HSet(key, strconv.FormatInt(aid, 10), retStr)
    dblogic.R("Games").Expire(key, time.Second * 6 * 3600)
    dblogic.R("Games").ZAdd(okeys.Landing("ACHIEVE"), redis.Z{
        Score:  float64(time.Now().Unix()),
        Member: strconv.FormatInt(user.Uid, 10),
    })
    return
}

//更新玩家成就信息
func (ach *Achievement) UpdateAchi(user *UserInfo, aid int64, num int64, opt string) bool {
    ret := ach.GetUserAchieveByAid(user, aid)
    if opt == "+" {
        ret.Process = ret.Process + num
    } else if  opt == "=" {
        ret.Process = num
    }
    ach.UpdateUserAchieve(user, aid, ret)
    return true
}
