package games

import (
    //"fmt"
    "time"
    "strconv"
    //"reflect"
    //"database/sql"

    "Crazy_Push/models/log"
    "Crazy_Push/models/dblogic"
    "Crazy_Push/models/tables"
    "Crazy_Push/models/okeys"
    "Crazy_Push/core/function"

    "github.com/go-redis/redis"
)

type Attr struct { }

func NewAttr() *Attr {
    return &Attr{}
}

//通过Gid获取物品信息
func (attr *Attr) Get(user *UserInfo) map[string]int64 {
    key := okeys.UserAttr(user.Uid)
    tmp, _ := dblogic.R("Games").HGetAll(key)
    arr := make(map[string]int64)
    if len(tmp) == 0 {
        ret := attr.recoverData(user)
        if !ret {
            return arr
        }
        return attr.Get(user)
    }
    for k, v := range tmp {
        vv, _ := strconv.ParseInt(v, 10, 64)
        arr[k] = vv
    }
    return arr
}

//从用户表中恢复数据到缓存
func (attr *Attr) recoverData(user *UserInfo) bool {
    gameInfo := &tables.GhGameinfo{}
    dblogic.M(dblogic.User).Where("mid = ?", user.Uid).First(&gameInfo)
    if gameInfo == nil {
        return false
    }
    tmp := function.Struct2Map(*gameInfo)
    arr := make(map[string]interface{})
    keys := GetAttrs()
    for _, key := range keys {
        arr[key] = tmp[key]
    }
    _, err := dblogic.R().HMSet(okeys.UserAttr(user.Uid), arr)
    if err != nil {
        return false
    }
    return true
}

//增加
func (attr *Attr) Add(user *UserInfo, flag string, count int64, rid int64, reason string) int64 {
    if _, ok := attrMap[flag]; !ok {
        log.LogError("attr_err", "变更资源数量属性类别错误:"+flag, user.Uid)
        return -1
    }
    num := attr.change(user, flag, count)
    if num <= 0 {
        return -1
    }
    //写流水,成就，活动
    CrazyCls.Currency.Log(user, flag, "+", count, num, rid, reason)
    if flag == "money" {
        CrazyCls.Achievement.UpdateAchi(user, 1, count, "+") //金币成就
        user.SaveStatistics("money", count) //个人基本统计(历史总获得金币数)
    }

    if flag == "star" {
        CrazyCls.Achievement.UpdateAchi(user, 6, count, "+") //星星成就
    }
    return num
}

//减少
func (attr *Attr) Del(user *UserInfo, flag string, count int64, rid int64, reason string) int64 {
    num := attr.change(user, flag, -count)
    if num < 1 {
        return num
    }
    //写流水,成就，活动
    CrazyCls.Currency.Log(user, flag, "-", count, num, rid, reason)
    if flag == "star" {
        CrazyCls.Achievement.UpdateAchi(user, 6, count, "-") //星星成就
    }
    return num
}

//更改用户属性数据(money,spins,diamond,star)
func (attr *Attr) change(user *UserInfo, flag string, count int64) int64 {
    key := okeys.UserAttr(user.Uid)
    num, _ := dblogic.R("Games").HIncrBy(key, flag, count)
    if num < 0 {
        dblogic.R("Games").HIncrBy(key, flag, -count) //回滚
        log.LogError("attr_err", "更改用户属性失败:"+flag, count, user.Uid)
        return 0
    }
    attr.JoinAttrLand(user)
    return int64(num)
}

//加入世界列表
func (attr *Attr) JoinAttrLand(user *UserInfo) (bool, error) {
    _, err := dblogic.R("Games").ZAdd(okeys.Landing("GAMEINFO"), redis.Z{
        Score:  float64(time.Now().Unix()),
        Member: strconv.FormatInt(user.Uid, 10),
    })
    if err != nil {
        return false, err
    }
    return true, nil
}
