package games

import (
    "strconv"
    "strings"

	jsoniter "github.com/json-iterator/go"
	"github.com/json-iterator/go/extra"
)

//涉及比较/计算变量/数据表整型字段,整型统一用int64,浮点型统一用float64

const (
    MONEY   = 1 //金币
    SPINS   = 2 //体力
    SHIELD  = 3 //盾牌
    STAR    = 4 //星星
    DIAMOND = 5 //钻石
)

//json数据类型兼容php的字符类型的数值
var json = jsoniter.ConfigCompatibleWithStandardLibrary
var (
    staticsKeys = []string{
        "money","damage","destroy",
        "coinbya","stealnum","allstealnum",
        "stealcoin","defense","damaged",
        "destroyed","stealed","loginTime",
    }
    attr    = []string{"money", "spins", "shield", "star", "diamond"} //玩家资源属性字段
    attrMap = map[string]int64{"money": MONEY, "spins": SPINS, "shield": SHIELD, "star": STAR, "diamond": DIAMOND} //玩家资源属性字段

    CrazyCls *CommonCls //初始化一个不需要重复实例化的实例类列表
)

type CommonCls struct {
    Attr        *Attr //资源属性
    Currency    *Currency //流水
    Achievement *Achievement //成就
    Config      *Config
}

func init() {
    //兼容php json string(float/int)等类型
    extra.RegisterFuzzyDecoders()

    CrazyCls = NewCommonCls()
}

func NewCommonCls() *CommonCls {
    return &CommonCls{
        Attr: NewAttr(),
        Currency: NewCurrency(),
        Config: NewConfig(),
    }
}

func GetAttrs() []string {
    return attr
}

func CompareVersion(v1 string, v2 string, args ... bool) bool {
    v1Arr := strings.Split(v1, ".")
    v2Arr := strings.Split(v2, ".")
    var length int
    if len(v1Arr) > len(v2Arr) {
        length = len(v1Arr)
    } else {
        length = len(v2Arr)
    }
    var v1Num, v2Num = 0, 0
    var err error
    for i := 0; i < length; i++ {
        if v1Arr[i] == "" {
            v1Num = 0
        }else{
            v1Num, err = strconv.Atoi(v1Arr[i])
            if err != nil {
                return false
            }
        }
        if v2Arr[i] == "" {
            v2Num = 0
        } else {
            v2Num, err = strconv.Atoi(v2Arr[i])
            if err != nil {
                return false
            }
        }

        if v1Num > v2Num {
            return true
        } else if v1Num < v2Num {
            return false
        }
    }

    if len(args) <= 0 || args[0] {
        return true
    }
    return false
}

//倍率
func Bet(bet int64) int64 {
    if bet < 0 {
        bet = -bet
    }
    if bet > 1000 {
        return 1000
    }
    return 1000
}

//int64类型数值大小比较
func CompareInt64(c1 int64, c2 int64) int64 {
    if c1 > c2 {
        return c1
    }
    return c2
}
