package games
//
import (
    "Crazy_Push/models/dblogic"
    "Crazy_Push/models/log"
    "Crazy_Push/models/okeys"
    "Crazy_Push/models/tables"
    "Crazy_Push/models/utils"
    "fmt"
    "strconv"
    "strings"
    "time"
)

type Config struct { }

type UpLimitConfData struct {
    LevelId int64 `json:"levelId"`
    UpLimit int64 `json:"upLimit"`
}

func NewConfig() *Config {
    return &Config{}
}

func (conf *Config) GetUpLimitConfig(flag string) []*UpLimitConfData {
    key := okeys.UpLimitConfig()
    tmp, _ := dblogic.R("Games").HGet(key, flag)
    tb := &tables.TUplimitConfig{}
    if tmp == "" {
        dblogic.M(dblogic.Config).Where("type = ?", flag).First(tb)
        if tb.Data != "" {
            _,_ = dblogic.R("Games").HSet(key, flag, tb.Data)
        }
        tmp = tb.Data
    }
    var data []*UpLimitConfData
    err := json.Unmarshal([]byte(tmp) ,&data)
    if err != nil {
        fmt.Println("解析level up limit config json数据失败:", err, tmp)
        log.LogError("member", "解析level up limit config json数据失败:", err, tmp)
    }
    return data
}

/**
* @Author Owen
* @Description coins 转化 商品ID
* @Date 10:22 2021/1/8
* @Param coins 字符
* @return 商品ID
**/
func (conf *Config) CoinsToGid (coins string) int64 {
    coinInt,_ := strconv.Atoi(strings.ReplaceAll(coins,"coins0",""))
    return int64(200 + coinInt)
}

/**
* @Author Owen
* @Description 商品ID 转化 coins
* @Date 10:26 2021/1/8
* @Param 商品ID
* @return coins 字符
**/
func (conf *Config) GidToCoins (coinInt int64) string {
    coin := "coins0" + string(coinInt - 200)
    return coin
}

/**
* @Author Owen
* @Description 根据签到日期获取分组签到配置
* @Date 12:07 2021/1/8
* @Param 
* @return 
**/

func (conf *Config) SignInConfByUserGroupOnId (uid int64,confId int64) * tables.SignInRewardConfig {
    confArr := conf.SignInConfByUserGroup(uid)
    return confArr[confId]
}

/**
* @Author Owen
* @Description 根据签到日期获取分组签到配置
* @Date 12:07 2021/1/8
* @Param
* @return
**/
func (conf *Config) SignInConfByUserGroupToClient (uid int64) [] * tables.SignInRewardConfigClient {
    levelId := NewUser(uid).GetUserAssetsInfo()["levelId"]
    confArr := conf.SignInConfByUserGroup(uid)
    var clientConfArr [] * tables.SignInRewardConfigClient
    for _,v := range confArr{
        curConf := &tables.SignInRewardConfigClient{}
        curConf.Id = v.Id
        var propsArr [] * tables.Props
        if v.Spins > 0 {
            p := &tables.Props{PropsId:15, Num:v.Spins}
            propsArr = append(propsArr,p)
        }
        bet,_ := strconv.ParseInt(v.Coins["bet"],64,10)
        if bet > 0 {
            p := &tables.Props{PropsId:14, Num:bet * GetGoodCoinsByLevelId(levelId,v.Coins["conis"])}
            propsArr = append(propsArr,p)
        }
        propsObj := NewProps()
        for _,j := range v.Props{
            value := propsObj.GetChestData(j.PropsId)
            if value != nil {
                j.Value = value
            }
            propsArr = append(propsArr,j)
        }
        clientConfArr = append(clientConfArr,curConf)
    }
    return clientConfArr
}

/**
* @Author Owen
* @Description 根据分组获取签到配置
* @Date 12:07 2021/1/8
* @Param
* @return
**/
func (conf *Config) SignInConfByUserGroup (uid int64) map[int64]*tables.SignInRewardConfig {
    cacheKey := okeys.SignInUserGroupId(uid)
    cache,_  := dblogic.R("games").Get(cacheKey)
    var id int64
    var config map[int64]*tables.SignInRewardConfig
    setCache := false
    if cache == "" {
        setCache    = true
        groupId    := NewUser(uid).GetUserDynamicsData().GroupId2
        groupIdArr := utils.ArrayFilter(strings.Split(groupId,","))
        if len(groupIdArr) == 1 {
            id , _ = strconv.ParseInt(groupIdArr[0],64,10)
        }else{
            id = 0
        }
    }else{
        id , _ = strconv.ParseInt(cache,64,10)
    }
    if id != 0 {
        config = conf.GetUserGroupConfigById(id).SignIn
    }
    if len(config) == 0 {
        setCache = true
        id = 0
        _ = json.Unmarshal([]byte(conf.GetSysCon("signIn","default")),&config)
    }
    if setCache {
        _,_ = dblogic.R("").Set(cacheKey,id,7 * 24 * time.Hour)
    }
    return config
}

/**
* @Author Owen
* @Description 判断是否已领取版本奖励
* @Date 13:08 2021/1/8
* @Param uid
* @return bool
**/
func (conf *Config) GetVersionGiftStatus(uid int64) bool {
    userVersion := NewUser(uid).GetUserInfo().Version
    userVersion  = utils.ChangeVersionLength(userVersion,3)
    tmp, _ := dblogic.R("games").Get(okeys.VersionGiftLog(uid,userVersion))
    if tmp != "" {
        return true
    }
    obj := &tables.TVersionGiftLog{}
    dblogic.M("statistics").Where("uid = ? AND version LIKE ?",uid,"%"+userVersion+"%").First(&obj)
    if obj.Id != 0 {
        return  false
    }
    _,_ = dblogic.R("games").Set(okeys.VersionGiftLog(uid,userVersion),1,3 * 24 * time.Hour)
    return true
}

/**
* @Author Owen
* @Description 根据用户分组ID获取配置
* @Date 19:48 2021/1/11
**/
func (conf *Config) GetUserGroupConfigById(id int64) *tables.TUserGroupRedis {
    cacheKey:= okeys.UserGroupConfig()
    cache,_ := dblogic.R("games").HGet(cacheKey,string(id))
    objRedis:= &tables.TUserGroupRedis{}
    if cache == "" {
        obj := &tables.TUserGroup{}
        between := map[string]string{"4":"point", "5" :"levelId", "6" :"active", "7" :"onlineTime", "8" :"regTime", "9" :"friendNum", "10" :"friendRank", "11" :"countryRank"}
        dblogic.M("Config").Where("id = ? ",id).First(&obj)
        objRedis.TUserGroup = obj
        if obj.Id != 0 {
            _ = json.Unmarshal([]byte(obj.Condition),&objRedis.Condition)
            _ = json.Unmarshal([]byte(obj.SignIn),&objRedis.SignIn)
            betweenInterFace := utils.ArrayStringToInterFace(utils.ArrayKeys(between))
            for i,v := range objRedis.Condition{
                if utils.InArray(v["condition"],betweenInterFace) && v["condition"] != "0" {
                    key := v["condition"].(string)
                    objRedis.Condition[i][between[key]] = strings.Split(v[between[key]].(string),".")
                }
            }
            objRedisStr ,_ := json.Marshal(objRedis)
            _,_ = dblogic.R("games").HSet(cacheKey,string(id),objRedisStr)
        }
    }else{
        err := json.Unmarshal([]byte(cache),&objRedis)
        if err != nil {

        }
    }
    return objRedis
}

/**
* @Author Owen
* @Description 获取全部用户分组配置
* @Date 19:48 2021/1/11
**/
func (conf *Config) GetUserGroupConfigByArr() []*tables.TUserGroupRedis {
    cacheKey:= okeys.UserGroupConfig()
    cache,_ := dblogic.R("games").HGet(cacheKey,"0")
    var objRedisArr []*tables.TUserGroupRedis
    if cache == "" {
        var objArr  []*tables.TUserGroup
        between := map[string]string{"4":"point", "5" :"levelId", "6" :"active", "7" :"onlineTime", "8" :"regTime", "9" :"friendNum", "10" :"friendRank", "11" :"countryRank"}
        dblogic.M("Config").Find(&objArr)
        betweenInterFace := utils.ArrayStringToInterFace(utils.ArrayKeys(between))
        if len(objArr) != 0 {
            for _,obj := range objArr {
                objRedis := &tables.TUserGroupRedis{}
                objRedis.TUserGroup = obj
                _ = json.Unmarshal([]byte(obj.Condition),&objRedis.Condition)
                _ = json.Unmarshal([]byte(obj.SignIn),&objRedis.SignIn)
                for i,v := range objRedis.Condition{
                    if utils.InArray(v["condition"],betweenInterFace) && v["condition"] != "0" {
                        key := v["condition"].(string)
                        objRedis.Condition[i][between[key]] = strings.Split(v[between[key]].(string),".")
                    }
                }
                objRedisArr = append(objRedisArr,objRedis)
            }
            objRedisArrStr ,_ := json.Marshal(objRedisArr)
            _,_ = dblogic.R("games").HSet(cacheKey,string("0"),objRedisArrStr)
        }
    }else{
        err := json.Unmarshal([]byte(cache),&objRedisArr)
        if err != nil {

        }
    }
    return objRedisArr
}

/**
* @Author Owen
* @Description 系統配置
* @Date 16:36 2021/1/14
* @原php方法 getSysCon
**/
func (conf *Config) GetSysCon(item string,title string) string {
    if item == "" || title == ""{
        return ""
    }
    cache,_ := dblogic.R("games").HGet(okeys.SystemConfig(),item + "-" + title)
    if cache == "" {
        obj := &tables.TSystemConfig{}
        dblogic.M("config").Where("item = ? AND title = ?",item,title).First(&obj)
        if obj.Content != ""{
            _,_ = dblogic.R("games").HSet(okeys.SystemConfig(),item + "-" + title,obj.Content)
            cache = obj.Content
        }
    }
    return cache
}

/**
* @Author Owen
* @Description 宝箱配置
* @Date 16:36 2021/1/14
* @原php方法 GetChestConfig
**/
func (conf *Config) GetChestConfig(id int64) [] *tables.TChestConfigReward {
    cacheKey := okeys.ChestConfig(id)
    cache,_  := dblogic.R("games").Get(cacheKey)
    var objRedis [] * tables.TChestConfigReward
    if cache == "" {
        obj := &tables.TChestConfig{}
        dblogic.M("config").Where("propsId = ?",id).First(&obj)
        if obj.PropsId == 0 {
            // TODO 抛异常
        }
        _,_ = dblogic.R("games").Set(cacheKey,obj.Reward,7 * 24 * time.Hour)
        _ = json.Unmarshal([]byte(obj.Reward),&objRedis)
    }else{
        _ = json.Unmarshal([]byte(cache),&objRedis)
    }
    return objRedis
}

/**
* @Author Owen
* @Description 分享配置
* @Date 16:42 2021/1/14
* @原php方法 getShareConfig
**/
func (conf *Config) GetShareConfig() [] *tables.TShareConfigReward {
    cacheKey := okeys.ShareConfig()
    cache,_  := dblogic.R("games").HGetAll(cacheKey)
    var confArr [] *tables.TShareConfigReward
    if len(cache) != 0 {
        for _,v := range cache {
            tempObj := &tables.TShareConfig{}
            _ = json.Unmarshal([]byte(v),&tempObj)
            rewardObj := &tables.TShareConfigReward{TShareConfig:tempObj}
            var reward []*tables.Props
            _ = json.Unmarshal([]byte(tempObj.Reward),&reward)
            rewardObj.Reward = reward
            confArr = append(confArr,rewardObj)
        }
    }else{
        var tempObj []tables.TShareConfig
        dblogic.M("config").Find(&tempObj)
        for _,v := range tempObj {
            tempString ,_ := json.Marshal(v)
            _,_ = dblogic.R("games").HSet(cacheKey,v.Type,string(tempString))
            rewardObj := &tables.TShareConfigReward{TShareConfig:&v}
            var reward []*tables.Props
            _ = json.Unmarshal([]byte(v.Reward),&reward)
            rewardObj.Reward = reward
            confArr = append(confArr,rewardObj)
        }
    }
    return confArr
}

/**
* @Author Owen
* @Description 分享配置
* @Date 16:42 2021/1/14
* @原php方法 getShareConfig
**/
func (conf *Config) GetShareConfigByType(typeName string) (*tables.TShareConfigReward,bool) {
    for _,v := range conf.GetShareConfig() {
        if v.Type == typeName {
            return v,true
        }
    }
    return nil,false
}

/**
* @Author Owen
* @Description 分享配置
* @Date 16:42 2021/1/14
* @原php方法 getVersionConfig
**/
func (conf *Config) GetVersionConfig(uid int)  {
    // TODO 涉及workerman取数据
}

/**
* @Author Owen
* @Description 系統補償
* @Date 16:42 2021/1/14
* @原php方法 getVersionConfig
**/
func (conf *Config) GetSysCompensateConfig(uid int) *tables.TSyscompenConfigData {
    cacheKey := okeys.SysCompensateConfig()
    cache,_  := dblogic.R("games").HGetAll(cacheKey)
    rObj     := &tables.TSyscompenConfigData{}
    if len(cache) > 0 {
        rObj.Id , _ = strconv.ParseInt(cache["id"],64,10)
        rObj.Reward = cache["reward"]
        rObj.EndTime ,    _ = strconv.ParseInt(cache["endTime"],64,10)
        rObj.LevelId ,    _ = strconv.ParseInt(cache["levelId"],64,10)
        rObj.LevelIdEnd , _ = strconv.ParseInt(cache["levelIdEnd"],64,10)
        return rObj
    }
    dbObj := &tables.TSyscompenConfig{}
    dblogic.M("Config").Order("id DESC").First(&dbObj)
    if dbObj.Id > 0 {
        utils.CopyStruct(dbObj,rObj)
        _,_ = dblogic.R("games").HMSet(cacheKey, map[string]interface{}{
            "id" : dbObj.Id,
            "reward" : dbObj.Reward,
            "endTime" : dbObj.EndTime,
            "levelId" : dbObj.LevelId,
            "levelIdEnd" : dbObj.LevelIdEnd,
        })
    }
    return nil
}

/**
* @Author Owen
* @Description 偷打活动配置
* @Date 16:42 2021/1/14
* @原php方法 getStealAttackConfig
**/
func (conf *Config) GetStealAttackConfig(stage int) * tables.TStealattack {
    cacheKey := okeys.StealAttackConfig()
    cache,_ := dblogic.R("games").HGet(cacheKey,string(stage))
    dbObj := &tables.TStealattack{}
    if cache != "" {
        _ = json.Unmarshal([]byte(cache),&dbObj)
        return dbObj
    }
    dblogic.M("Config").Where("stage = ?",stage).First(&dbObj)
    s ,_ := json.Marshal(dbObj)
    _ ,_ = dblogic.R("games").HSet(cacheKey,string(stage),string(s))
    return dbObj
}

/**
* @Author Owen
* @Description 建筑大师配置
* @Date 11:29 2021/1/18
* @原php方法 getBuildMasterConfig
**/
func (conf *Config) GetBuildMasterConfig(levelId int) * tables.TBuildingmasterConfig {
    cacheKey := okeys.BuildMasterConfig()
    cache,_ := dblogic.R("games").HGet(cacheKey,string(levelId))
    dbObj := &tables.TBuildingmasterConfig{}
    if cache != "" {
        _ = json.Unmarshal([]byte(cache),&dbObj)
        return dbObj
    }
    dblogic.M("Config").Where("levelId = ?",levelId).First(&dbObj)
    s ,_ := json.Marshal(dbObj)
    _ ,_ = dblogic.R("games").HSet(cacheKey,string(levelId),string(s))
    return dbObj
}

/**
* @Author Owen
* @Description 矿工小游戏配置
* @Date 11:29 2021/1/18
* @原php方法 GetMiningGameConfig
**/
func (conf *Config) GetMiningGameConfig() map[int64]*tables.TMiningGameConfigRedis {
    cacheKey := okeys.MiningGameConfig()
    cache,_ := dblogic.R("games").HGetAll(cacheKey)
    redisObjArr := make(map[int64]*tables.TMiningGameConfigRedis)
    if len(cache) > 0 {
        redisObj := &tables.TMiningGameConfigRedis{}
        for _,v := range cache{
            _ = json.Unmarshal([]byte(v),&redisObj)
            if redisObj.Lv > 0 {
                redisObjArr[redisObj.Lv] = redisObj
            }
        }
    }else{
        var dbObjArr [] *tables.TMiningGameConfig
        dblogic.M("Config").Find(&dbObjArr)
        var mineralData []*tables.MineralData
        var hookUpSpeed []*tables.HookUpSpeed
        for _,v := range dbObjArr{
            _ = json.Unmarshal([]byte(v.MineralData),&mineralData)
            _ = json.Unmarshal([]byte(v.HookUpSpeed),&hookUpSpeed)
            redisObj := &tables.TMiningGameConfigRedis{TMiningGameConfig:v,MineralData:mineralData,HookUpSpeed:hookUpSpeed}
            redisObjArr[redisObj.Lv] = redisObj
            s ,_ := json.Marshal(&redisObj)
            _ ,_ = dblogic.R("games").HSet(cacheKey,string(redisObj.Lv),s)
        }
    }
    return redisObjArr
}

/**
* @Author Owen
* @Description 矿工小游戏配置
* @Date 11:29 2021/1/18
* @原php方法 GetMiningGameConfig
**/
func (conf *Config) GetMiningGameConfigByLevelId(levelId int64) *tables.TMiningGameConfigRedis {
    arr := conf.GetMiningGameConfig()
    for _,v := range arr {
        if v.Lv == levelId {
            return v
        }
    }
    return nil
}

/**
* @Author Owen
* @Description 获取倍率活动配置
* @Date 11:29 2021/1/18
* @原php方法 GetMiningGameConfig
**/
func (conf *Config) GetCrazyBet() []*tables.TCrazyBetConfig {
    var dbObjArr []*tables.TCrazyBetConfig
    dblogic.M("Config").Find(&dbObjArr)
    return dbObjArr
}

/**
* @Author Owen
* @Description 签到进度配置
* @Date 11:29 2021/1/18
* @原php方法 getSignInProgressConfig
**/
func (conf *Config) GetSignInProgressConfig(sectionId int64) map[int64]*tables.TSignInProgressConfigRedis {
    cacheKey := okeys.SignInProgressConfig(sectionId)
    cache,_ := dblogic.R("games").HGetAll(cacheKey)
    redisObjArr := make(map[int64]*tables.TSignInProgressConfigRedis)
    if len(cache) > 0 {
        redisObj := &tables.TSignInProgressConfigRedis{}
        for _,v := range cache{
            _ = json.Unmarshal([]byte(v),&redisObj)
            if redisObj.Progress > 0 {
                redisObjArr[redisObj.Progress] = redisObj
            }
        }
    }else{
        var dbObjArr [] *tables.TSignInProgressConfig
        dblogic.M("Config").Where("section_id = ?",sectionId).Find(&dbObjArr)
        var coins *tables.CoinsBet
        var props []*tables.Props
        for _,v := range dbObjArr{
            _ = json.Unmarshal([]byte(v.Coins),&coins)
            _ = json.Unmarshal([]byte(v.Props),&props)
            redisObj := &tables.TSignInProgressConfigRedis{Progress:v.Progress, Coins:coins, Spins:v.Spins, Props:props, Count:int64(len(dbObjArr))}
            redisObjArr[redisObj.Progress] = redisObj
            s ,_ := json.Marshal(&redisObj)
            _ ,_ = dblogic.R("games").HSet(cacheKey,string(redisObj.Progress),s)
        }
        _ ,_ = dblogic.R("games").Expire(cacheKey,7 * 24 * time.Hour)
    }
    return redisObjArr
}

/**
* @Author Owen
* @Description 签到进度配置
* @Date 11:29 2021/1/18
* @原php方法 getSignInProgressConfig
**/
func (conf *Config) GetSignInProgressConfigByProgress(sectionId int64,progress int64) *tables.TSignInProgressConfigRedis {
    arr := conf.GetSignInProgressConfig(sectionId)
    for _,v := range arr {
        if v.Progress == progress {
            return v
        }
    }
    return nil
}

/**
* @Author Owen
* @Description 签到进度配置
* @Date 11:29 2021/1/18
* @原php方法 RecallConfig
**/
func (conf *Config) RecallConfig() []*tables.TRecallRedis {
    cacheKey := okeys.RecallConfig()
    cache,_ := dblogic.R("games").HGetAll(cacheKey)
    var redisObjArr [] *tables.TRecallRedis
    if len(cache) > 0 {
        redisObj := &tables.TRecallRedis{}
        for _,v := range cache{
            _ = json.Unmarshal([]byte(v),&redisObj)
            redisObjArr = append(redisObjArr,redisObj)
        }
    }else{
        var dbObjArr [] *tables.TRecall
        dblogic.M("Config").Order("day").Find(&dbObjArr)
        var reward *tables.TRecallReward
        for i,v := range dbObjArr{
            _ = json.Unmarshal([]byte(v.Reward),&reward)
            redisObj := &tables.TRecallRedis{Day:v.Day, Reward:reward}
            redisObjArr = append(redisObjArr,redisObj)
            s ,_ := json.Marshal(&redisObj)
            _ ,_ = dblogic.R("games").HSet(cacheKey,string(i),s)
        }
        _ ,_ = dblogic.R("games").Expire(cacheKey,7 * 24 * time.Hour)
    }
    return redisObjArr
}

/**
* @Author Owen
* @Description 签到进度配置
* @Date 11:29 2021/1/18
* @原php方法 RecallConfig
**/
func (conf *Config) RecallConfigClient() []*tables.TRecallClient {
    arr := conf.RecallConfig()
    var clientObjArr [] *tables.TRecallClient
    for _,v := range arr {
        coins := &tables.CoinsGid{
            Gid:conf.CoinsToGid(v.Reward.Coins.Coins),
            Bet:v.Reward.Coins.Bet,
        }
        clientRewardObj := &tables.TRecallRewardClient{
            Coins:coins,
            Spins:v.Reward.Spins,
        }
        clientObj := &tables.TRecallClient{
            Day:v.Day,
            Reward:clientRewardObj,
        }
        clientObjArr = append(clientObjArr,clientObj)
    }
    return clientObjArr
}
