package games

import (
    "Crazy_Push/models/dblogic"
    "Crazy_Push/models/log"
    "Crazy_Push/models/okeys"
    "Crazy_Push/models/tables"
    "fmt"
    "github.com/go-redis/redis"
    "strconv"
    "time"
)

type LevelElement struct {
    Id       int64 `json:"id"`
    LevelId  int64 `json:"level"`
    MaxLevel int64 `json:"maxLevel"`
    Status   int64 `json:"status"`
}

type LevelInfo struct {
    LevelId int64 `json:"levelId"`
    IsReward int64 `json:"isReward"`
    Element []*LevelElement `json:"element"`
}

type UserInfo struct {
    //Attr    *Attr
    Uid     int64
    VipInfo *NewVipInfo
    //Diamond int64
    //Money   int64
    //Spins   int64
    //Star    int64

    //LevelInfo *LevelInfo
    //...
}

//初始化用户
//初始化玩家资源信息,不写内存，直接从redis获取
func NewUser(uid int64) *UserInfo {
    return &UserInfo{ Uid: uid, VipInfo: NewVip() }
}

//获取用户等级
func (user *UserInfo) GetUserLevel() int64 {
    key := okeys.UserBuildLevel(user.Uid)
    info, _ := dblogic.R("Games").HGetAll(key)
    if len(info) == 0 { //没有取到数据，从数据表中获取
        levelInfo := user.GetUserLevelInfo()
        return levelInfo.LevelId
    }
    if _, ok := info["levelId"]; !ok {
        fmt.Println("get user level fail1", user.Uid)
        log.LogError("member", "get user level fail1", user.Uid)
        return 0
    }
    level, err := strconv.Atoi(info["levelId"])
    if err != nil {
        fmt.Println("parse user from str to int fail", user.Uid)
        log.LogError("member", "parse user from str to int fail", user.Uid)
        return 0
    }
    return int64(level)
}

//获取用户等级信息
func (user *UserInfo) GetUserLevelInfo() *LevelInfo {
    key := okeys.UserBuildLevel(user.Uid)
    tmp, _ := dblogic.R("Games").HGetAll(key)
    element := ""
    levelInfo := &LevelInfo{}
    if len(tmp) == 0 { //没有取到数据，从数据表中获取
        tb := &tables.TUserlevel{}
        tname := tb.TableName()+strconv.Itoa(int(user.Uid) % 100)
        dblogic.M(dblogic.UserLevel).
            Table(tname).
            Where("uid = ?", user.Uid).
            Order("levelId DESC").First(&tb)

        if tb.Uid == 0 {
            fmt.Println("未找到玩家"+strconv.Itoa(int(user.Uid))+"的等级数据")
            log.LogError("member", "未找到玩家的等级数据", user.Uid)
            return levelInfo
        }

        element = tb.Element
        set := make(map[string]interface{}, 3)
        set["is_reward"] = tb.IsReward
        set["levelId"] = tb.LevelId
        set["element"] = tb.Element
        _, err := dblogic.R("Games").HMSet(key, set)
        if err != nil {
            dblogic.R("Games").Expire(key, time.Second * 24 * 3600)
        }
        levelInfo.LevelId = tb.LevelId
        levelInfo.IsReward = tb.IsReward
    }else{
        element = tmp["element"]
        tlevelId, _ := strconv.Atoi(tmp["levelId"])
        tIsReward, _ := strconv.Atoi(tmp["is_reward"])
        levelInfo.LevelId = int64(tlevelId)
        levelInfo.IsReward = int64(tIsReward)
    }

    err := json.Unmarshal([]byte(element), &levelInfo.Element)
    if err != nil {
        fmt.Println("解析level info json数据失败:", user.Uid, err, element)
        log.LogError("member", "解析level info json数据失败:", user.Uid, err, element)
    }
    return levelInfo
}

//获取玩家信息(金币,体力,星星,钻石)
func (user *UserInfo) GetUserAssetsInfo() map[string]int64 {
    return CrazyCls.Attr.Get(user)
}

//获取玩家信息信息
func (user *UserInfo) GetUserInfo() *tables.TUserinfo {
    key := okeys.UserInfo(user.Uid)
    tmp, _ := dblogic.R("Games").Get(key)
    info := &tables.TUserinfo{}
    if tmp == "" { //没有取到数据，从数据表中获取
        dblogic.M(dblogic.User).Where("uid = ?", user.Uid).First(&info)
        if user.Uid == 0 {
            fmt.Println("未找到玩家"+strconv.Itoa(int(user.Uid))+"信息")
            log.LogError("member", "未找到玩家信息", user.Uid)
            return info
        }
        resStr, _ := json.Marshal(info)
        dblogic.R("Games").Set(key, string(resStr), time.Second*7*24*60*60)
    } else {
        err := json.Unmarshal([]byte(tmp), &info)
        if(err != nil) {
            fmt.Println("解析user info json数据失败:", user.Uid, err, info)
            log.LogError("member", "解析user info json数据失败:", user.Uid, err, info)
            return info
        }
    }
    return info
}

//更新用户属性
func (user *UserInfo) OptProperty(arr map[string]int64, t string, rid int64, reason string) (bool, int64) {
    var res int64
    for k, v := range arr {
        if _, ok := attrMap[k]; !ok {
            continue
        }
        if t == "+" {
            res = CrazyCls.Attr.Add(user, k, v, rid, reason)
        } else {
            res = CrazyCls.Attr.Del(user, k, v, rid, reason)
        }
    }
    if res < 0 {
        return false, -1
    }
    return true, res
}

//获取个人流水统数据计
func(user *UserInfo) GetStatistics(flag string) {
    key := okeys.UserBasicsStatistics(user.Uid)
    res, _ := dblogic.R("Games").HGetAll(key)
    if len(res) < 0 || len(res) < len(staticsKeys) {
        user.recoverUserBasicsStatistics()
    }
    return
}

//恢复用户基本统计(获取钱money，打伤了demage，被打伤数demaged，打破次数destory，被打破次数destoryed，打获取的总钱coinbya
//偷次数stealnum，完美偷次allstealCnum，偷到总钱stealcoin，被偷总钱数stealed，防护次数defense)
func (user *UserInfo) recoverUserBasicsStatistics() {
    ////////////////////////////////////////////////
    ////////////////////////////////////////////////
}

//写入流水统计缓存
func (user *UserInfo) SaveStatistics(flag string, num int64) bool {
    user.GetStatistics(flag) //恢复可能没有的数据

    _, err := dblogic.R("Games").HIncrBy(okeys.UserBasicsStatistics(user.Uid), flag, num)
    if err != nil {
        return false
    }

    //数据落地标识
    dblogic.R("Games").ZAdd(okeys.Landing("STATISTICS"), redis.Z{
        Score:  float64(time.Now().Unix()),
        Member: strconv.FormatInt(user.Uid, 10),
    })

    return true
}

func (user *UserInfo) GetUserLevelUpLimit(from string) int64 {
    levelId := user.GetUserLevel()
    if from == "shield" {
        max := user.GetLevelUpLimit(levelId, from)
        var vipShield int64 = 3
        //if user.VipInfo.CheckVip(user) {
        //    //config    = oo::commonOprModel('newvip')->getUserVipLv($uid, true);
        //    //vipShield += intval($config['shieldGroove']);
        //}
        if max > vipShield {
            return max
        }
        return vipShield
    }
    if from == "recovery" {
        //max := user.GetLevelUpLimit(levelId, "recovery") // 恢复多少
        //$checkVip = oo::commonOprModel('newvip')->checkVip($uid);
        //$setSpins = oo::commonOprModel('readconfig')->getCon('basic','restoreSpins');; // 恢复多少
        //if($checkVip){
        //    if(oo::compareVersion(oo::commonOprModel('user')->getUserVersion($uid),'1.3.5')) {
        //        $config = oo::commonOprModel('newvip')->getUserVipLv($uid,true,false,155);
        //        $setSpins += intval($config['spinsRestore']);
        //    }else{
        //        $setSpins += intval(oo::commonOprModel('readconfig')->getCon('vip','refreshSpins'));
        //    }
        //}
        //return max($levelSpins,$setSpins,5);
    }
    return 0
}

//获取上限配置
func (user *UserInfo) GetLevelUpLimit(levelId int64, from string) int64 {
    conf := CrazyCls.Config.GetUpLimitConfig(from)
    var min int64
    for _, v := range conf {
        if levelId <= v.LevelId {
            return v.UpLimit
        }
        min = v.UpLimit
    }
    return min
}

//func (user *UserInfo) GetUserDynamicsData () *tables.TUserdynamics {
//    cacheKey := okeys.UserDynamicsData(user.Uid)
//    cache,_  := dblogic.R("games").Get(cacheKey)
//    obj      := &tables.TUserdynamics{}
//    if cache != ""{
//        err  := json.Unmarshal([]byte(cache),&obj)
//        if err != nil{
//
//        }
//    }else{
//        dblogic.M("user").Where("uid = ?",user.Uid).First(&obj)
//        if obj.Uid == 0{
//
//        }else{
//            ret,_ := json.Marshal(obj)
//            dblogic.R("games").Set(cacheKey,string(ret),3 * 24 * time.Hour)
//        }
//    }
//    return obj
//}


/**
 * 获取用户动态数据
 * @param $uid
 * @return array|false|mixed
 * Created by: Owen
 * Created on: 2020/7/23 14:37
 */
func (user *UserInfo) GetUserDynamicsData() *tables.TUserdynamics {
    cacheKey   := okeys.UserDynamicsData(user.Uid)
    tmp, _ := dblogic.R("Games").Get(cacheKey) //获取缓存数据
    tb  := &tables.TUserdynamics{}
    if tmp == "" {
        err := dblogic.M("USER").Where("uid = ?", user.Uid).First(tb).Error
        if err != nil {
            log.LogError("member", "get user dynamic data error", user.Uid)
            tb.Uid = user.Uid
            dblogic.M("USER").Create(tb) //插入表数据
            dblogic.M("USER").Where("uid = ?", user.Uid).First(tb)
        }
        ret, _ := json.Marshal(tb)
        dblogic.R("Games").Set(cacheKey, string(ret), time.Second * 7 * 24 * 3600) //更新缓存数据
    } else {
        err := json.Unmarshal([]byte(tmp), tb)
        if err != nil {
            log.LogError("member", "unmarshal user dynamic data error", user.Uid, err)
        }
    }
    return tb
}


