package games

import (
    "strconv"
    "fmt"
    //"encoding/base64"
    //"math/rand"
    "time"
    //"math"
	//"encoding/json"
    ////"strings"
    "Crazy_Push/models/dblogic"
    "Crazy_Push/models/okeys"
    "Crazy_Push/models/tables"
    "Crazy_Push/models/log"
)

type NewVipInfo struct { }

func NewVip() *NewVipInfo {
    return &NewVipInfo{}
}

type VipLvConf struct {
    Lv                  int64 `json:"lv"`
    Integration         int64 `json:"integration"`
    SignInBet           int64 `json:"signInBet"`
    TaskRewardBet       int64 `json:"taskRewardBet"`
    SpinsRestore        int64 `json:"spinsRestore"`
    AvatarBox           int64 `json:"avatarBox"`
    ShieldGroove        int64 `json:"shieldGroove"`

    TurnTable           map[string]int64 `json:"turnTable"`
    ShopAppend          string `json:"shopAppend"`
    WeekRewardSpins     int64  `json:"weekRewardSpins"`
    WeekRewardShield    int64  `json:"weekRewardShield"`

    WeekRewardChest     map[string]int64 `json:"weekRewardChest"`
    WeekRewardBless     map[string]int64 `json:"weekRewardBless"`
    WeekRewardLuckydraw map[string]int64 `json:"weekRewardLuckydraw"`
    DayVipPoint         int64 `json:"dayVipPoint"`
    AttackCoinsPro      int64 `json:"attackCoinsPro"`
    StealCoinsPro       int64 `json:"stealCoinsPro"`
    AttackCss           int64 `json:"attackCss"`
    MiningGame          int64 `json:"miningGame"`
    Mock                int64 `gorm:"json:"mock"`
    OneClick            int64 `json:"oneClick"`
    RichkingStyle       int64 `json:"richkingStyle"`
}

//获取玩家vip等级
func (vip *NewVipInfo) GetUserVipLv(user *UserInfo, fromCache bool, version string) int64 {
    if fromCache {
        info := user.GetUserInfo()
        return info.VipLv
    }
    return 0
}

//获取玩家vip等级配置信息
func (vip *NewVipInfo) GetUserVipLvConf(user *UserInfo, version string) (vipConf *VipLvConf) {
    curPoint := vip.GetUserCurPoint(user)
    if curPoint == 0 {
        return
    }
    vipConf = vip.GetVipLvConfig(curPoint, version)
    return
}

//获取玩家vip信息
func (vip *NewVipInfo) GetUserVipLvInfo(user *UserInfo) int64 {
    return 0
}

//获取玩家vip积分
func (vip *NewVipInfo) GetUserCurPoint(user *UserInfo) int64 {
    info := user.GetUserInfo()
    point := info.VipPoint
    if point == -1 {
        //$totalPayNum = oo::commonOprModel('payment')->getUserTotalPayNum($uid);
        //$vipPoint = intval($totalPayNum*100);
        //if($vipPoint != 0 && $vipPoint<100){
        //    $vipPoint = 150;
        //}
        //$updateInfo['svipPoint'] = $vipPoint;
        //oo::commonOprModel('member')->getUserInfo($uid, NULL, true,$updateInfo);
    }
    return point
}

//判断是否vip玩家
func (vip *NewVipInfo) CheckVip(user *UserInfo) (bool, map[string]int64) {
    info := user.GetUserInfo()
    uidStr := strconv.FormatInt(user.Uid, 10)
    ret := make(map[string]int64)
    if info.VipExpire > time.Now().Unix() {
        version := user.GetUserVersion()
        userVipConf := vip.GetUserVipLvConf(user, version)
        if userVipConf == nil {
            ret["level"]    = 0
            ret["curPoint"] = 0
            vTmp, _ := dblogic.R("Games").HGet(okeys.VipTurnTable(), uidStr)
            if vTmp != "" {
                vCacheLv, err := strconv.ParseInt(vTmp, 10, 64)
                if err != nil {
                    log.LogError("newvip_err", "check vip error", user.Uid, vTmp)
                    fmt.Println("check vip error", user.Uid, vTmp)
                    vCacheLv = 0
                }
                ret["tpStatus"] = CompareInt64(3 - vCacheLv, 0)
            }
            return true, ret
        }
        curPoint := vip.GetUserCurPoint(user)
        var vipLv int64 = 0
        if curPoint != 0 && info.VipLv == 0 {
            ///////////////////
            //updateData = []
            ///////////////////
            vipLv = userVipConf.Lv
        }
        if info.VipExperienceLv != vipLv {
            ///////////////////
            //updateData = []
            ///////////////////
        }
        ret["level"] = vipLv
        ret["curPoint"] = curPoint

        vTmp, _ := dblogic.R("Games").Get(okeys.VipExperience(user.Uid))
        if vTmp != "1" {
            ret["isFreeing"] = 1
        } else {
            ret["isFreeing"] = 0
        }

        ///////////////////
        return true, ret
        ///////////////////
    }

    if info.Vip == 0 {
        return false, ret
    }
    return false, ret
}

//获取vip配置数据
func (vip *NewVipInfo) GetVipLvConfig(integration int64, version string) *VipLvConf {
    key    := okeys.VipLvConfig(version)
    tCache, _ := dblogic.R("Games").HGetAll(key)
    ret := &VipLvConf{}
    if len(tCache) > 0 {
        for i := 0; i < len(tCache); i++ {
            tmp := &VipLvConf{}
            err := json.Unmarshal([]byte(tCache[strconv.Itoa(i)]), tmp)
            if err != nil {
                log.LogError("newvip_err", "解析vip等级配置json错误", tCache[strconv.Itoa(i)])
                fmt.Println("解析vip等级配置json错误", tCache[strconv.Itoa(i)])
                return nil
            }
            if tmp.Integration < integration {
                ret = tmp
            }
        }
        return ret
    }
    tb := &tables.TVipLvConfig{}
    var tname = tb.TableName()
    if version != "" {
        tname = tname+"2"
    }
    var lvConfig []*tables.TVipLvConfig
    dblogic.M(dblogic.Config).Table(tname).Find(&lvConfig)
    strArr := make(map[string]interface{})
    for k, v := range lvConfig {
        WeekRewardChest := make(map[string]int64)
        WeekRewardBless := make(map[string]int64)
        WeekRewardLuckydraw := make(map[string]int64)
        TurnTable := make(map[string]int64)
        json.Unmarshal([]byte(v.WeekRewardChest), &WeekRewardChest)
        json.Unmarshal([]byte(v.WeekRewardBless), &WeekRewardBless)
        json.Unmarshal([]byte(v.WeekRewardLuckydraw), &WeekRewardLuckydraw)
        json.Unmarshal([]byte(v.TurnTable), &TurnTable)
        tmp := &VipLvConf {
            Lv:                  v.Lv,
            Integration:         v.Integration,
            SignInBet:           v.SignInBet,
            TaskRewardBet:       v.TaskRewardBet,
            SpinsRestore:        v.SpinsRestore,
            AvatarBox:           v.AvatarBox,
            ShieldGroove:        v.ShieldGroove,

            TurnTable:           TurnTable,
            ShopAppend:          v.ShopAppend,
            WeekRewardSpins:     v.WeekRewardSpins,
            WeekRewardShield:    v.WeekRewardShield,

            WeekRewardChest:     WeekRewardChest,
            WeekRewardBless:     WeekRewardBless,
            WeekRewardLuckydraw: WeekRewardLuckydraw,
            DayVipPoint:         v.DayVipPoint,
            AttackCoinsPro:      v.AttackCoinsPro,
            StealCoinsPro:       v.StealCoinsPro,
            AttackCss:           v.AttackCss,
            MiningGame:          v.MiningGame,
            Mock:                v.Mock,
            OneClick:            v.OneClick,
            RichkingStyle:       v.RichkingStyle,
        }
        if tmp.Integration < integration && ret.Integration == 0 {
            ret = tmp
        }
        byteStr, _ := json.Marshal(tmp)
        strArr[strconv.Itoa(k)] = string(byteStr)
    }
    if len(strArr) > 0 {
        dblogic.R("Games").HMSet(key, strArr)
        dblogic.R("Games").Expire(key, time.Second * 7 * 24 * 3600)
    }
    return ret
}
