package games
//
import (
    "Crazy_Push/models/dblogic"
    "Crazy_Push/models/okeys"
    "Crazy_Push/models/tables"
    "time"
)

type ReadConfig struct { }

type HotFix struct {
    Version string `json:"version"`
    IsForce int64 `json:"isForce"`
}

func NewReadConfig() *ReadConfig {
    return &ReadConfig{}
}

//拉取静态配置
func (conf *ReadConfig) GetStaticConfigPath(sid int64) map[string]string {
    key := okeys.StaticConfig()
    ret, err := dblogic.R("Games").HGetAll(key)
    if err != nil || len(ret) == 0 {
        ret = map[string]string{}
        staticConfs := []*tables.TStaticConfig{}
        dblogic.M("CONFIG").Find(&staticConfs)
        ret = map[string]string{}
        for _, val := range staticConfs {
            ret[val.Config] = val.Path
            dblogic.R("Games").HSet(key, val.Config, val.Path)
        }
    }
    if sid == 1 {
        ret["Version"] = ret["AndroidVersion"]
    } else if sid == 2 {
        ret["Version"] = ret["IosVersion"]
    }
    delete(ret, "AndroidVersion")
    delete(ret, "IosVersion")
    delete(ret, "VivoVersion")
    return ret
}

//静态配置MD5值
func (conf *ReadConfig) GetStaticMd5ConfigPath() map[string]string {
    key := okeys.StaticMd5Config()
    ret, err := dblogic.R("Games").HGetAll(key)
    if err == nil && len(ret) > 0 {
        return ret
    }
    staticMd5Confs := []*tables.TStaticMd5Config{}
    dblogic.M("CONFIG").Find(&staticMd5Confs)
    ret = map[string]string{}
    for _, val := range staticMd5Confs {
        ret[val.Config] = val.Path
        dblogic.R("Games").HSet(key, val.Config, val.Path)
    }
    return ret
}

//获取语言静态配置
func (conf *ReadConfig) GetLangStaticConfigPath() map[string]string {
    key := okeys.LangStaticConfig()
    ret, err := dblogic.R("Games").HGetAll(key)
    if err == nil && len(ret) > 0 {
        return ret
    }
    staticLangConfs := []*tables.TCliLangConfig{}
    dblogic.M("CONFIG").Find(&staticLangConfs)
    ret = map[string]string{}
    for _, val := range staticLangConfs {
        ret[val.Config] = val.Path
        dblogic.R("Games").HSet(key, val.Config, val.Path)
    }
    return ret
}

func (conf *ReadConfig) GetGameUpdateHotfix(sid int64, version string) *HotFix {
    key := okeys.GameUpdateHotFix(sid, version)
    hotFix, err := dblogic.R("Games").Get(key)
    hotFixRet := &HotFix{}
    if err == nil && hotFix != "" {
        err := json.Unmarshal([]byte(hotFix), hotFixRet)
        if err == nil {
            return hotFixRet
        }
    }

    var device = "android"
    if sid != 1 {
        device = "ios"
    }

    d := &tables.Gameupdate{}
    err = dblogic.M("ADMIN").Where("gu_version LIKE ? AND gu_device=?", "%"+version+"%", device).First(d).Error

    hotFixRet.Version = version
    hotFixRet.IsForce = 0

    if err == nil && d.Id != 0 {
        upData := &tables.Gamehotfix{}
        err = dblogic.M("ADMIN").Where("id=?", d.Id).First(upData).Error
        if err == nil  && upData.Id != 0 {
            hotFixRet.Version = upData.Version
            hotFixRet.IsForce = upData.IsForce
        }
    }
    resStr, _ := json.Marshal(hotFixRet)
    dblogic.R("Games").Set(key, resStr, 7 * 86400 * time.Second)
    return hotFixRet
}
