package games

import (
    "fmt"
    "time"
    "runtime"

	"runtime/debug"
    "sync"
    "strconv"

    "Crazy_Push/models/log"

	"github.com/gorilla/websocket"

    cmap "github.com/orcaman/concurrent-map"
)

var (
    clientManager = NewClientManager()
	pingPeriod    = int64(60)
)

// 连接管理
type ClientManager struct {
	Clients     cmap.ConcurrentMap // 全部的连接
	ClientsLock sync.RWMutex       // 读写锁
	Join        chan *Join         // 用户连接处理
	Leave       chan *Client       // 断开连接处理程序
	Broadcast   chan []byte        // 广播 向全部成员发送数据
}

type Join struct {
    Addr string
	Uid int64
	Client *Client
}

func NewClientManager() (clientManager *ClientManager) {
    clientManager = &ClientManager{
		Clients:   cmap.New(),
        Leave:     make(chan *Client, 1000),
		Join:      make(chan *Join, 1000),
		Broadcast: make(chan []byte, 1000),
    }
    return
}

func InitWebSocket(uid int64, socket *websocket.Conn) {
    client := clientManager.EventJoin(uid, socket)
	defer clientManager.EventLeave(client)

	go client.write()
	client.read()
}

//验证token是否正确(暂不验证token)
func VerifyToken(uid int64, token string) bool {
    /////////////////////////////////
    //uidInt, _ := strconv.Atoi(uid)
    //tmp := dblogic.R("Games").Get(key, okeys.UserToken(uidInt))
    //if tmp.Val() != token {
    //    return false
    //}
    //fmt.Println("kkkkkkk", key)
    /////////////////////////////////
    return true
}

//加入websocket连接列表
func (manager *ClientManager) EventJoin(uid int64, socket *websocket.Conn) *Client {
	client := NewClient(uid, socket.RemoteAddr().String(), socket, time.Now().Unix())
	manager.Clients.Set(strconv.FormatInt(uid,10), client)
    return client
}

//获取所有客户端连接
func (manager *ClientManager) GetClients() cmap.ConcurrentMap {
    return manager.Clients
}

// 遍历
func (manager *ClientManager) ClientsRange(f func(value *Client) (result bool)) {
    for item := range manager.Clients.IterBuffered() {
        client := item.Val.(*Client)
        if f(client) == false {
            return
        }
    }
    return
}

//获取当前总连接数
func (manager *ClientManager) GetClientsLen() int {
    return manager.Clients.Count()
}

//客户端断开连接
func (manager *ClientManager) EventLeave(client *Client) {
    manager.Clients.Remove(strconv.FormatInt(client.Uid,10))
}

//获取用户的连接
func (manager *ClientManager) GetClient(uid int64) (client *Client) {
    strUid := strconv.FormatInt(uid,10)
    c, ret := manager.Clients.Get(strUid)
    if ret {
        client = c.(*Client)
    }
	return
}

//判断玩家是否在线
func CheckOldUser(uid int64) bool {
    c, ret := clientManager.Clients.Get(strconv.FormatInt(uid,10))
    if ret {
        c.(*Client).SendMsg([]byte("已在其他设备登录"))
        clientManager.EventLeave(c.(*Client))
        return true
    }
    return false
}

//获取用户的uid列表
func (manager *ClientManager) GetUidList() (userList []int64) {
	userList = make([]int64, 0)
    for item := range manager.Clients.IterBuffered() {
        client := item.Val.(*Client)
        userList = append(userList, client.Uid)
    }

	fmt.Println("GetUidList len:", len(manager.Clients))
    log.LogInfo("websocket", "GetUidList len:", len(manager.Clients))
	return
}

//向全部成员(除了自己)发送数据
func (manager *ClientManager) sendAll(message []byte, ignoreClient *Client) {
	for item := range manager.Clients.IterBuffered() {
        conn := item.Val.(*Client)
		if conn != ignoreClient && conn != nil {
            conn.SendMsg(message)
		}
	}
}

//管道处理程序
func (manager *ClientManager) start() {
	for {
		select {
		case join := <-manager.Join: // 客户端建立连接
			manager.EventJoin(join.Uid, join.Client.Socket)
		case conn := <-manager.Leave: // 断开连接事件
			manager.EventLeave(conn)
		case message := <-manager.Broadcast:
			// 广播事件
            for item := range manager.Clients.IterBuffered() {
                conn := item.Val.(*Client)
				select {
				case conn.Send <- message:
				default:
					close(conn.Send)
				}
			}
		}
	}
}

/**************************  manager info  ***************************************/
//获取管理者信息
func GetManagerInfo(isDebug string) (managerInfo map[string]interface{}) {
	managerInfo = make(map[string]interface{})

	managerInfo["clientsLen"] = clientManager.GetClientsLen()        // 客户端连接数
	managerInfo["chanJoinLen"] = len(clientManager.Join)     // 未处理连接事件数
	managerInfo["chanLeaveLen"] = len(clientManager.Leave) // 未处理退出登录事件数
	managerInfo["chanBroadcastLen"] = len(clientManager.Broadcast)   // 未处理广播事件数

	if isDebug == "true" {
		addrs := make([]string, 0)
        users := make([]int64, 0)
		clientManager.ClientsRange(func(client *Client) (result bool) {
			addrs = append(addrs, client.Addr)
			users = append(users, client.Uid)
			return true
		})

		managerInfo["clients"] = addrs // 客户端列表
		managerInfo["users"] = users // 登录用户列表
	}
	return
}

////定时清理超时连接
//func ClearTimeoutConnections() {
//	currentTime := time.Now().Unix()
//	clientManager.ClientsLock.RLock()
//
//	for _, client := range clientManager.Clients {
//		if client.IsHeartbeatTimeout(currentTime) {
//			fmt.Println("心跳时间超时 关闭连接", client.Addr, client.Uid, client.HeartbeatTime)
//            log.LogError("websocket", "心跳时间超时 关闭连接", client.Addr, client.Uid, client.HeartbeatTime)
//            clientManager.EventLeave(client)
//		}
//	}
//	clientManager.ClientsLock.RUnlock()
//}
//
//获取全部用户
func GetUserList() (userList []int64) {
	fmt.Println("获取全部用户id列表")
	userList = clientManager.GetUidList()
	return
}

// 全员广播
func AllSendMessages(uid int64, data string) {
	fmt.Println("全员广播", uid, data)
	ignoreClient := clientManager.GetClient(uid)
	clientManager.sendAll([]byte(data), ignoreClient)
}

//=============== Client ================//
//用户连接
type Client struct {
	Addr          string          // 客户端地址
	Socket        *websocket.Conn // 用户连接
	Send          chan []byte     // 待发送的数据
	Uid           int64           // 用户Id，用户登录以后才有
	FirstTime     int64           // 首次连接事件
	HeartbeatTime int64           // 用户上次心跳时间
}

//初始化
func NewClient(uid int64, addr string, socket *websocket.Conn, firstTime int64) (client *Client) {
	client = &Client{
		Addr:          addr,
        Uid:           uid,
		Socket:        socket,
		Send:          make(chan []byte, 100),
		FirstTime:     firstTime,
		HeartbeatTime: firstTime,
	}
	return
}

//读取客户端数据
func (c *Client) read() {
	defer func() {
		if r := recover(); r != nil {
			fmt.Println("write stop", string(debug.Stack()), r)
		}
		close(c.Send)
        clientManager.EventLeave(c)
		fmt.Println("读取客户端数据 关闭发送", c)
        log.LogError("websocket", "读取客户端数据 关闭发送", c)
	}()

	for {
		_, message, err := c.Socket.ReadMessage()
		if err == nil {
            // 处理程序
            fmt.Println("读取客户端数据 处理:", string(message))
            //////////////////////////
            c.SendMsg(message) //调试阶段,数据原样返回
            //ProcessData(c, message)
            //////////////////////////
            c.Heartbeat(time.Now().Unix())
            continue
        }

        if websocket.IsUnexpectedCloseError(err, websocket.CloseGoingAway) {
            //fmt.Println("error: %v", err)
            log.LogError("websocket", "玩家非正常退出", err)
        }
        return
	}
}

//向客户端写数据
func (c *Client) write() {
	ticker := time.NewTicker(time.Duration(pingPeriod) * time.Second)
	defer func() {
		if r := recover(); r != nil {
			fmt.Println("write stop", string(debug.Stack()), r)
            log.LogError("websocket", "write stop", string(debug.Stack()), r)
		}
		ticker.Stop()
        fmt.Println("通道关闭", c.Uid, c.Addr)
        log.LogError("websocket", "通道关闭", c.Uid, c.Addr)
        clientManager.EventLeave(c)
        runtime.GC() //强制调用gc回收
	}()

	for {
		select {
		case message, ok := <-c.Send:
			if !ok {
                fmt.Println("Client发送数据 关闭连接", c.Addr)
                log.LogError("websocket", "Client发送数据 关闭连接", c.Addr)
				return //发送数据错误关闭连接
			}

            err := c.Socket.WriteMessage(websocket.TextMessage, message)
            if err != nil {
                c.Heartbeat(time.Now().Unix())
            }
        case <-ticker.C:
            if c.IsHeartbeatTimeout(time.Now().Unix()){
				fmt.Println("心跳超时", c.Addr)
                log.LogError("websocket", "心跳超时", c.Addr)
                return
            }
            err := c.Socket.WriteMessage(websocket.TextMessage, []byte(c.PingMsg()))
            if err != nil {
				fmt.Println("ping pong error", err)
                log.LogError("websocket", "ping pong error", err)
                return
            }
		}
	}
}

//ping-pong
func (c *Client) PingMsg() string {
    return RspMsg("userHeartbeatRec", 1, "", "")
}

//ping-pong
func (c *Client) PongMsg() string {
    return RspMsg("userHeartbeat", 1, "", "")
}

//读取客户端数据
func (c *Client) SendMsg(msg []byte) {
	defer func() {
		if r := recover(); r != nil {
			fmt.Println("SendMsg stop:", r, string(debug.Stack()))
            log.LogError("websocket", "SendMsg stop:", r, string(debug.Stack()))
		}
	}()

	if c == nil {
		return
	}

	c.Send <- msg
}

//读取客户端数据
func (c *Client) close() {
	close(c.Send)
}

//用户心跳
func (c *Client) Heartbeat(currentTime int64) {
	c.HeartbeatTime = currentTime
	return
}

//心跳超时
func (c *Client) IsHeartbeatTimeout(currentTime int64) (timeout bool) {
	if c.HeartbeatTime+pingPeriod <= currentTime {
		timeout = true
	}
	return
}
