package nats

import (
    "fmt"
    //"sync"
	"os"

    "Crazy_Push/configs"
    "Crazy_Push/core/function"
    //"time"

    nats "github.com/nats-io/nats.go"
	"github.com/nats-io/stan.go"
)

var (
    natsConf = sysConfig.GetServerConf()
    natsHttpServe *natsServer
    //////////////////////////////////
    natsTopic = "test.b"
    //////////////////////////////////
    natsChan  = make(chan [2]string, 50) // Channel for new join users.
	interrupt = make(chan os.Signal, 1)
)

// 默认配置
type natsServer struct {
    Conn stan.Conn
}

func NewNatsEvent(strArr [2]string) {
    natsChan <- strArr
}

func NatsChanProduce() {
    if sysConfig.MQOPEN == 0 {
        return
    }
	for {
		select {
		case strArr := <-natsChan:
            go natsHttpServe.Produce(strArr[0], strArr[1])
		case <-interrupt:
            close(natsChan)
            natsHttpServe.Close()
            return //退出
		}
	}
}

//给http服务设定一个nats连接服务
func HttpNats() *natsServer {
    if natsHttpServe == nil {
        natsHttpServe = NewPubServer()
    }
    if natsHttpServe.Conn == nil {
        natsHttpServe = NewPubServer()
    }
    return natsHttpServe
}


func InitHttpNats() {
    HttpNats() //初始化http nats server
}

func NewPubServer () *natsServer {
    return &natsServer { Conn: natsConnect("pub") }
}

//主题
func GetTopic() string {
    return natsTopic
}

//订阅者
func NewSubServer() *natsServer {
    return &natsServer { Conn: natsConnect("sub") }
}

func natsConnect(flag string) stan.Conn {
    sfx := function.Ucfirst(flag)
    nc, err := nats.Connect(natsConf["natsServer"], nats.UserInfo(natsConf["nats"+sfx], natsConf["nats"+sfx+"Pwd"]))
    if err != nil {
        return nil
    }
    client := "test-client"+flag
    stnc, _ := stan.Connect("test-cluster", client, stan.NatsConn(nc))
    return stnc
}

// nats发布消息
func (server *natsServer) Produce(msgBody string, topic string) error {
    err := server.Conn.Publish(topic, []byte(msgBody))
    if err != nil && server.Conn == nil { //发布失败,重新连接
        server.Conn = natsConnect("pub")
        if server.Conn == nil {
            fmt.Println("连接失败", err)
            return err
        }
        err = server.Conn.Publish(topic, []byte(msgBody))
    }
    return err
}

//关闭订阅
func (server *natsServer) Close() {
    server.Close()
}

/**
 * 消费
 */
func (server *natsServer) Consume(topic string) {
    server.Conn.Subscribe(topic, func(m *stan.Msg) {
        fmt.Printf("Received a message: %s, published time:%d\n", string(m.Data), int64(m.Timestamp))
    //}, stan.DeliverAllAvailable())
    //}, stan.StartAtTimeDelta(500 * time.Second))
    //}, stan.StartWithLastReceived())
    }, stan.DurableName(GetTopic())) //按主题来接收最后一次收到的数据
}
