package statistics

import (
	"fmt"
	"strconv"
	"strings"
	"time"

	"Crazy_Push/models/dblogic"
	"Crazy_Push/models/log"
	"Crazy_Push/models/okeys"
	"Crazy_Push/models/tables"
)

const (
	GOOGLECHANNEL     = "google"
	FACEBOOKCHANNEL   = "facebook"
	IRONSOURCECHANNEL = "ironsource"
	UNITYCHANNEL      = "unity"
	APPLOVINCHANNEL   = "applovin"
	CHARTBOOSTCHANNEL = "chartboost"
	MINTEGRALCHANNEL  = "mintegral"
	TIKTOKCHANNEL     = "tiktok"
	APPLECHANNEL      = "apple"

	EVENTADCOSTNAME  = "ad_cost"
	EVENTUSERSETNAME = "user_set"

	ADJUSTTANUM = 500
)

//adjust callbacks传过来的买量用户记录
type AdsUser struct{}

//处理ajdust 1002上报事件
type AdjustSta struct {
	RdsFlag string //redis单例实例化标志
}

//type AjustTaReport struct {
//    AccountId  int `json:"accountId"`
//    DistinctId string `json:"distinctId"`
//    EventName  string `json:"eventName"`
//    Properties map[string]interface{} `json:"properties"`
//}

type AdsTaReport struct {
	RdsFlag string
}

type AdjustTaReportData struct {
	AccountId  int             `json:"accountId"`
	DistinctId string          `json:"distinctId"`
	EventName  string          `json:"eventName"`
	Properties AdsTaReportData `json:"properties"`
}

//network_name#adjust_network_name渠道campaign_name#adjust_campaign_name广告系列adgroup_name#adjust_adgroup_name广告组creative_name#adjust_creative_name素材名称

type AdsTaReportData struct {
	NetworkId    string `json:"network_id"`              //渠道
	NetworkName  string `json:"network_name,omitempty"`  //渠道
	CampaignId   string `json:"campaign_id"`             //广告系列
	CampaignName string `json:"campaign_name"`           //广告系列名
	AdgroupId    string `json:"adgroup_id,omitempty"`    //广告组id
	AdgroupName  string `json:"adgroup_name,omitempty"`  //广告组名
	CreativeId   string `json:"creative_id,omitempty"`   //广告id(素材id)
	CreativeName string `json:"creative_name,omitempty"` //广告id(素材名)

	UserSource int `json:"user_source,omitempty"`

	Cost     float64 `json:"cost,omitempty"`
	Clicks   int64   `json:"clicks,omitempty"`
	Installs int64   `json:"installs,omitempty"`
	Time     string  `json:"#time,omitempty"`
	EventId  string  `json:"#event_id,omitempty"`

	Country string `json:"country,omitempty"`
}

func NewAdsUser() *AdsUser {
	return &AdsUser{}
}

func NewAdjustSta() *AdjustSta {
	return &AdjustSta{RdsFlag: "AdjustSta"}
}

//广告成本数据上报数数平台
func NewAdsTaReport() *AdsTaReport {
	return &AdsTaReport{RdsFlag: "AdsTaReport"}
}

//更新当天新买量用户
func (ads *AdsUser) SaveAdsUser(dayStamp int64) bool {
	adData := []*tables.TAdjustData{}
	sqlStr := "created_at >= ? AND created_at < ? AND network_name IN('Google Ads UAC','Facebook Installs','Off-Facebook Installs','Facebook Messenger Installs','unity','Instagram Installs')"

	err := dblogic.M("ADJUSTSTA").Where(sqlStr, dayStamp, dayStamp+86400).Find(&adData).Error
	if err != nil || len(adData) < 1 {
		return true //记录不存在
	}

	gpsAdidArr := map[string]string{}
	inArr := []string{}
	for _, val := range adData {
		gpsAdidArr[val.GpsAdid] = val.NetworkName + "|" + val.CreatedAt
		if val.GpsAdid == "" {
			continue
		}
		inArr = append(inArr, val.GpsAdid)
	}

	adSta := []*tables.TAdjustSta{}
	tb := &tables.TAdjustSta{}
	var flag, gpsAdid, networkName, createdAt string = "", "", "", ""
	for i := 0; i < 100; i++ {
		tname := GetCircleTableName(tb.TableName(), int64(i))
		err = dblogic.M("ADJUSTSTA").Table(tname).Where("ios IN(?) OR android IN(?)", inArr, inArr).Find(&adSta).Error
		if err != nil || len(adSta) < 1 {
			continue
		}
		for _, val := range adSta {
			if val.Ios != "" {
				flag = "ios"
				gpsAdid = val.Ios
			} else {
				flag = "android"
				gpsAdid = val.Android
			}
			if _, ok := gpsAdidArr[gpsAdid]; !ok {
				networkName = ""
				createdAt = ""
			} else {
				tmp := strings.Split(gpsAdidArr[gpsAdid], "|")
				networkName = tmp[0]
				if len(tmp) < 2 {
					createdAt = ""
				} else {
					createdAt = tmp[1]
				}
			}
			//插入买量表
			dblogic.M("STATISTICS").Create(&tables.TAdjustAdsData{
				Uid:         val.Uid,
				Device:      flag,
				GpsAdid:     gpsAdid,
				NetworkName: networkName,
				CreatedAt:   createdAt,
			})
		}
	}
	return true
}

//10012上报落地到crazyGod_api_adjuststa.t_adjust_sta_*表
func (asta *AdjustSta) AsyncAdjustSta() {
	sids := []int{IOSSID, ANDROIDSID}
	for {
		tb := &tables.TAdjustSta{}
		for _, sid := range sids {
			sqlArr := map[string][]tables.TAdjustSta{}
			eventId, _ := strconv.ParseInt(ADVERTISEID, 10, 64)
			key := okeys.Report(eventId, int64(sid))

			strArr, err := dblogic.R("STATISTICS", asta.RdsFlag).HGetAll(key)
			if err != nil {
				fmt.Println("sid获取数据错误", err)
				return
			}
			if len(strArr) <= 0 {
				continue //没有获取到数据
			}
			userStrArr := map[string][]string{} //需要删除的uid列表
			redisDelArr := []string{}
			userDeviceArr := map[string]string{} //需要删除的uid列表
			for uidStr, val := range strArr {
				uidInt64, _ := strconv.ParseInt(uidStr, 10, 64)
				cacheKey := okeys.AdjustDeviceId(uidInt64)
				cacheRet, _ := dblogic.R("GAMES", asta.RdsFlag).Get(cacheKey)
				if cacheRet != "" { //已经更新过的忽略,减少重复入库
					redisDelArr = append(redisDelArr, uidStr)
					continue
				}
				userDeviceArr[uidStr] = val

				var atb *tables.TAdjustSta
				if sid == ANDROIDSID {
					atb = &tables.TAdjustSta{Uid: uidInt64, Android: val}
				} else {
					atb = &tables.TAdjustSta{Uid: uidInt64, Ios: val}
				}
				tName := GetCircleTableName(tb.TableName(), uidInt64)
				if _, ok := sqlArr[tName]; !ok {
					sqlArr[tName] = []tables.TAdjustSta{*atb}
					userStrArr[tName] = []string{uidStr}
				} else {
					sqlArr[tName] = append(sqlArr[tName], *atb)
					userStrArr[tName] = append(userStrArr[tName], uidStr)
				}
			}

			//当天上报数数一次平台
			if len(userDeviceArr) > 0 {
				asta.dealAndSendToTa(strArr, sid)
			}
			for name, val := range sqlArr {
				//批量插入,重复则忽略
				err := dblogic.M("ADJUSTSTA").Table(name).Clauses(dblogic.MySqlIgnore()).Create(val).Error
				if err != nil {
					log.LogError("async_adjuststa_log", "multi insert int adjust_sta error", err)
				}
				//删除uid列表
				dblogic.R("STATISTICS", asta.RdsFlag).HDel(key, userStrArr[name]...)
			}
			if len(redisDelArr) > 0 { //删除重复的uid列表
				dblogic.R("STATISTICS", asta.RdsFlag).HDel(key, redisDelArr...)
			}
		}
		time.Sleep(time.Second * 10) //实时性不要求那么高
	}
}

//批量处理并上报数数平台
func (asta *AdjustSta) dealAndSendToTa(uidDeviceArr map[string]string, sid int) {
	deviceUidArr := map[string]string{}
	deviceArr := []string{}
	for uidStr, deviceId := range uidDeviceArr {
		if len(deviceUidArr) < ADJUSTTANUM {
			deviceUidArr[deviceId] = uidStr
			deviceArr = append(deviceArr, deviceId)
			continue
		}
		asta.adjustTaReport(deviceUidArr, deviceArr, sid) //adjust数据数数上报
		deviceArr = []string{}                            //取出数据后,清空内存数据
		deviceUidArr = map[string]string{}                //上报完成,清空内存数据
	}
	if len(deviceArr) > 0 {
		asta.adjustTaReport(deviceUidArr, deviceArr, sid) //adjust数据数数上报
	}
}

func (asta *AdjustSta) adjustTaReport(deviceUidArr map[string]string, deviceArr []string, sid int) {
	eventKey := okeys.TaEvent(EVENTUSERSETNAME)
	var key string = "idfa"
	var uidStr = ""
	var deviceId = ""
	adata := []*tables.TAdjustData{}
	if sid == ANDROIDSID { //ios
		key = "gps_adid"
	}

	taReportData := &AdjustTaReportData{
		EventName: EVENTUSERSETNAME,
	}

	err := dblogic.M("ADJUSTSTA").Where(key+" IN(?)", deviceArr).Find(&adata).Error
	if err != nil {
		log.LogError("async_adjustta_log", "get ta device list err", err)
		return
	}
	for _, info := range adata {
		if sid == ANDROIDSID {
			uidStr = deviceUidArr[info.GpsAdid]
			deviceId = info.GpsAdid
		} else {
			uidStr = deviceUidArr[info.Idfa]
			deviceId = info.Idfa
		}
		if uidStr == "" {
			log.LogError("async_adjustta_log", "get uid by device list err")
			continue
		}
		uidInt, _ := strconv.Atoi(uidStr)
		cacheKey := okeys.AdjustDeviceId(int64(uidInt))

		taReportData.Properties = asta.getAdjustTaData(info)
		taReportData.AccountId = uidInt
		str, _ := json.Marshal(taReportData)
		//fmt.Println("上报数数数据:", string(str))
		dblogic.R("STATISTICS", asta.RdsFlag).RPush(eventKey, str)
		dblogic.R("GAMES", asta.RdsFlag).Set(cacheKey, deviceId, RedisRandomExpire(24*3600))
	}
}

//获取adjust数据,拼装数数上报结构
func (asta *AdjustSta) getAdjustTaData(info *tables.TAdjustData) AdsTaReportData {
	properties := AdsTaReportData{
		NetworkName:  info.NetworkName,
		CampaignName: info.CampaignName,
		AdgroupName:  info.AdgroupName,
		CreativeName: info.CreativeName,
	}
	properties.CampaignId = asta.getAdjustId(info.CampaignName)
	properties.AdgroupId = asta.getAdjustId(info.AdgroupName)
	properties.CreativeId = asta.getAdjustId(info.CreativeName)
	properties.UserSource = 0
	if strings.Contains(info.NetworkName, "Organic") { //自然用户
		properties.NetworkId = "organic"
	} else if strings.Contains(info.NetworkName, "Facebook") { //facebook
		properties.NetworkId = "facebook"
		properties.UserSource = 2
	} else if strings.Contains(info.NetworkName, "unity") { //unity
		properties.NetworkId = "unity"
		properties.UserSource = 2
	} else if strings.Contains(info.NetworkName, "ironSource") { //ironSource
		properties.NetworkId = "ironSource"
		properties.UserSource = 2
	} else if strings.Contains(info.NetworkName, "Google") {
		properties.NetworkId = "google"
		properties.UserSource = 2
	}
	if info.Label != "" {
		properties.UserSource = 1
	}
	return properties
}

func (asta *AdjustSta) getAdjustId(str string) string {
	preIndex := strings.LastIndex(str, "(")
	sufxIndex := strings.LastIndex(str, ")")
	if preIndex == -1 || sufxIndex == -1 {
		return ""
	}
	return str[preIndex+1 : sufxIndex]
}

//广告成本数据
func (adsta *AdsTaReport) AsyncAdsTaReport(n int) {
	var key string
	var err error
	day := GetDaysBeforeDate(n) //获取n天前的数据
	//google成本
	data := map[string]*AdsTaReportData{}
	googleData := []*tables.TGoogleCampaign{}
	err = dblogic.M("CAMPAIGN").Where("date_start = ?", day).Find(&googleData).Error
	if err != nil {
		log.LogError("async_campaign_log", "get google campaign data error", err)
	} else {
		for _, val := range googleData {
			//key = val.CampaignId+"_"+val.CampaignName
			key = val.CampaignId + "_" + val.Country + "_" + day
			if _, ok := data[key]; !ok {
				data[key] = &AdsTaReportData{
					NetworkId:    GOOGLECHANNEL,
					CampaignId:   val.CampaignId,
					CampaignName: val.CampaignName + " (" + val.CampaignId + ")",
					Installs:     val.Conversions,
					Clicks:       val.Clicks,
					Cost:         val.Spend,
					Time:         day + " 00:00:00",
					EventId:      day + "_" + key,
					Country:      val.Country,
				}
			} else {
				data[key].Cost = data[key].Cost + val.Spend
				data[key].Clicks = data[key].Clicks + val.Clicks
				data[key].Installs = data[key].Installs + val.Conversions
			}
		}
	}

	//facebook成本
	fbData := []*tables.TFacebookAds{}
	err = dblogic.M("CAMPAIGN").Where("date_start = ?", day).Find(&fbData).Error
	if err != nil {
		log.LogError("async_campaign_log", "get facebook campaign data error", err)
	} else {
		for _, val := range fbData {
			key = val.CampaignId + "_" + val.AdsetId + "_" + val.AdId + "_" + val.Country + "_" + day
			if _, ok := data[key]; !ok {
				data[key] = &AdsTaReportData{
					NetworkId:    FACEBOOKCHANNEL,
					CampaignId:   val.CampaignId,
					CampaignName: val.CampaignName + " (" + val.CampaignId + ")",
					AdgroupId:    val.AdsetId,
					AdgroupName:  val.AdsetName + " (" + val.AdsetId + ")",
					CreativeId:   val.AdId,
					CreativeName: val.AdName + " (" + val.AdId + ")",
					Installs:     val.AppInstall + val.MobileAppInstall,
					Clicks:       val.Clicks,
					Cost:         val.Spend,
					Time:         day + " 00:00:00",
					EventId:      day + "_" + key,
					Country:      val.Country,
				}
			} else {
				data[key].Cost = data[key].Cost + val.Spend
				data[key].Clicks = data[key].Clicks + val.Clicks
				data[key].Installs = data[key].Installs + val.AppInstall + val.MobileAppInstall
			}
		}
	}

	//unity成本
	unityData := []*tables.TUnityCampain{}
	err = dblogic.M("CAMPAIGN").Where("data_start = ?", day).Find(&unityData).Error
	if err != nil {
		log.LogError("async_campaign_log", "get unity campaign data error", err)
	} else {
		for _, val := range unityData {
			//key = val.Id+"_"+val.Name
			key = val.Id + "_" + val.Country + "_" + day
			if _, ok := data[key]; !ok {
				data[key] = &AdsTaReportData{
					NetworkId:    UNITYCHANNEL,
					CampaignId:   val.Id,
					CampaignName: val.Name,
					Installs:     val.Installs,
					Clicks:       val.Clicks,
					Cost:         val.Spend,
					Time:         day + " 00:00:00",
					EventId:      day + "_" + key,
					Country:      val.Country,
				}
			} else {
				data[key].Cost = data[key].Cost + val.Spend
				data[key].Clicks = data[key].Clicks + val.Clicks
				data[key].Installs = data[key].Installs + val.Installs
			}
		}
	}
	//ironsource成本
	ironData := []*tables.TIronsourceCampain{}
	err = dblogic.M("CAMPAIGN").Where("data_start = ?", day).Find(&ironData).Error
	if err != nil {
		log.LogError("async_campaign_log", "get ironsource campaign data error", err)
	} else {
		for _, val := range ironData {
			key = val.Id + "_" + val.Name + "_" + val.Country + "_" + day
			if _, ok := data[key]; !ok {
				data[key] = &AdsTaReportData{
					NetworkId:    IRONSOURCECHANNEL,
					CampaignId:   val.Id,
					CampaignName: val.Name,
					Installs:     val.Installs,
					Clicks:       val.Clicks,
					Cost:         val.Spend,
					Time:         day + " 00:00:00",
					EventId:      day + "_" + key,
					Country:      val.Country,
				}
			} else {
				data[key].Cost = data[key].Cost + val.Spend
				data[key].Clicks = data[key].Clicks + val.Clicks
				data[key].Installs = data[key].Installs + val.Installs
			}
		}
	}

	//applovin成本
	applovinData := []*tables.TApplovinCampain{}
	err = dblogic.M("CAMPAIGN").Where("data_start = ?", day).Find(&applovinData).Error
	if err != nil {
		log.LogError("async_campaign_log", "get applovin campaign data error", err)
	} else {
		for _, val := range applovinData {
			key = val.Id + "_" + "_" + val.Country + "_" + day
			if _, ok := data[key]; !ok {
				data[key] = &AdsTaReportData{
					NetworkId:    APPLOVINCHANNEL,
					CampaignId:   val.Id,
					CampaignName: val.Name + " (" + val.Id + ")",
					CreativeName: val.AdName,
					Installs:     val.Installs,
					Clicks:       val.Clicks,
					Cost:         val.Spend,
					Time:         day + " 00:00:00",
					EventId:      day + "_" + key,
					Country:      val.Country,
				}
			} else {
				data[key].Cost = data[key].Cost + val.Spend
				data[key].Clicks = data[key].Clicks + val.Clicks
				data[key].Installs = data[key].Installs + val.Installs
			}
		}
	}

	//chartboost成本
	chartboostData := []*tables.TChartBoostCampain{}
	err = dblogic.M("CAMPAIGN").Where("data_start = ?", day).Find(&chartboostData).Error
	if err != nil {
		log.LogError("async_campaign_log", "get chartboost campaign data error", err)
	} else {
		for _, val := range chartboostData {
			key = val.Id + "_" + "_" + val.Country + "_" + day
			if _, ok := data[key]; !ok {
				data[key] = &AdsTaReportData{
					NetworkId:    CHARTBOOSTCHANNEL,
					CampaignId:   val.Id,
					CampaignName: val.Name + " (" + val.Id + ")",
					CreativeName: val.AdName,
					Installs:     val.Installs,
					Clicks:       val.Clicks,
					Cost:         val.Spend,
					Time:         day + " 00:00:00",
					EventId:      day + "_" + key,
					Country:      val.Country,
				}
			} else {
				data[key].Cost = data[key].Cost + val.Spend
				data[key].Clicks = data[key].Clicks + val.Clicks
				data[key].Installs = data[key].Installs + val.Installs
			}
		}
	}

	//mintegral成本
	mintegralData := []*tables.TMintegralCampain{}
	err = dblogic.M("CAMPAIGN").Where("data_start = ?", day).Find(&mintegralData).Error
	if err != nil {
		log.LogError("async_campaign_log", "get mintegral campaign data error", err)
	} else {
		for _, val := range mintegralData {
			key = val.Id + "_" + val.AdGroupId + "_" + val.AdName + "_" + val.Country + "_" + day
			if _, ok := data[key]; !ok {
				data[key] = &AdsTaReportData{
					NetworkId:    MINTEGRALCHANNEL,
					CampaignId:   val.Id,
					CampaignName: val.Name + " (" + val.Id + ")",
					AdgroupId:    val.AdGroupId,
					AdgroupName:  val.AdGroupName,
					CreativeName: val.AdName,
					Installs:     val.Installs,
					Clicks:       val.Clicks,
					Cost:         val.Spend,
					Time:         day + " 00:00:00",
					EventId:      day + "_" + key,
					Country:      val.Country,
				}
			} else {
				data[key].Cost = data[key].Cost + val.Spend
				data[key].Clicks = data[key].Clicks + val.Clicks
				data[key].Installs = data[key].Installs + val.Installs
			}
		}
	}

	//tiktok成本
	tiktokData := []*tables.TTiktokCampain{}
	err = dblogic.M("CAMPAIGN").Where("data_start = ?", day).Find(&tiktokData).Error
	if err != nil {
		log.LogError("async_campaign_log", "get tiktok campaign data error", err)
	} else {
		for _, val := range tiktokData {
			key = val.CampainId + "_" + val.AdgroupId + "_" + val.AdId + "_" + val.Country + "_" + day
			if _, ok := data[key]; !ok {
				data[key] = &AdsTaReportData{
					NetworkId:    TIKTOKCHANNEL,
					CampaignId:   val.CampainId,
					CampaignName: val.CampainName + " (" + val.CampainId + ")",
					AdgroupId:    val.AdgroupId,
					AdgroupName:  val.AdgroupName,
					CreativeId:   val.AdId,
					CreativeName: val.AdName,
					Installs:     val.Installs,
					Clicks:       val.Clicks,
					Cost:         val.Spend,
					Time:         day + " 00:00:00",
					EventId:      day + "_" + key,
					Country:      val.Country,
				}
			} else {
				data[key].Cost = data[key].Cost + val.Spend
				data[key].Clicks = data[key].Clicks + val.Clicks
				data[key].Installs = data[key].Installs + val.Installs
			}
		}
	}

	//apple成本
	appleData := []*tables.TAppleCampain{}
	err = dblogic.M("CAMPAIGN").Where("data_start = ?", day).Find(&appleData).Error
	if err != nil {
		log.LogError("async_campaign_log", "get apple campaign data error", err)
	} else {
		for _, val := range appleData {
			key = val.CampainId + "_" + val.AdgroupId + "_" + val.Country + "_" + day
			if _, ok := data[key]; !ok {
				data[key] = &AdsTaReportData{
					NetworkId:    APPLECHANNEL,
					CampaignId:   val.CampainId,
					CampaignName: val.CampainName + " (" + val.CampainId + ")",
					AdgroupId:    val.AdgroupId,
					AdgroupName:  val.AdgroupName,
					Installs:     val.Installs,
					Clicks:       val.Clicks,
					Cost:         val.Spend,
					Time:         day + " 00:00:00",
					EventId:      day + "_" + key,
					Country:      val.Country,
				}
			} else {
				data[key].Cost = data[key].Cost + val.Spend
				data[key].Clicks = data[key].Clicks + val.Clicks
				data[key].Installs = data[key].Installs + val.Installs
			}
		}
	}

	trackData := AdjustTaReportData{}
	eventKey := okeys.TaEvent(EVENTADCOSTNAME)
	for k, v := range data {
		trackData = AdjustTaReportData{
			//AccountId: "admin",
			AccountId:  999,
			DistinctId: k,
			EventName:  EVENTADCOSTNAME,
			Properties: *v,
		}
		str, _ := json.Marshal(trackData)
		//fmt.Println("aaaaaaaaaaa", eventKey, string(str))
		dblogic.R("CAMPAIGN").RPush(eventKey, str)
	}
}
