package statistics

import (
	"fmt"
	"time"

	"Crazy_Push/models/dblogic"
	"Crazy_Push/models/log"
	"Crazy_Push/models/okeys"
	"Crazy_Push/models/tables"
)

const (
	ASSETSLOGPERNUM = 5000 //每次从流水日志中取出的数量
)

// 玩家流水日志入数据库
type AssetsLogs struct {
	RdsFlag string
}

// 玩家流水日志入库
func NewAssetsLogs() *AssetsLogs {
	return &AssetsLogs{
		RdsFlag: "AssetsLogs",
	}
}

// 流水当前小时之前一个小时的记录redis取出
func (assetLogs *AssetsLogs) AsyncAssetAfterHourLogs() {
	var num int64
	for {
		hour := GetDayAfterHour()
		fmt.Printf("hour:%+v\n", hour)
		key := okeys.AssetsLog(hour)
		strArr, err := dblogic.R("STATISTICS", assetLogs.RdsFlag).LRange(key, -ASSETSLOGPERNUM, -1)
		if err != nil {
			log.LogError("async_assets_log", "get data fail from redis", key)
			continue
		}
		if len(strArr) == 0 {
			log.LogInfo("async_assets_log_info", "前一个小时流水已跑完")
			time.Sleep(time.Second * 60)
			continue
		}
		if len(strArr) < int(ASSETSLOGPERNUM) {
			num = int64(len(strArr))
		} else {
			num = ASSETSLOGPERNUM
		}
		dblogic.R("STATISTICS", assetLogs.RdsFlag).LTrim(key, 0, -num-1) //从队列中删除
		assetLogs.strToDb(strArr)                                        //批量入库
		time.Sleep(time.Millisecond * 200)
	}
}

// 流水当前小时的记录redis取出
func (assetLogs *AssetsLogs) AsyncAssetLogs() {
	var num int64
	for {
		hour := GetDayHour()
		key := okeys.AssetsLog(hour)
		strArr, err := dblogic.R("STATISTICS", assetLogs.RdsFlag).LRange(key, -ASSETSLOGPERNUM, -1)
		if err != nil {
			log.LogError("async_assets_log", "get data fail from redis", key)
			continue
		}
		if len(strArr) == 0 {
			time.Sleep(time.Second * 5)
			continue
		}
		if len(strArr) < int(ASSETSLOGPERNUM) {
			num = int64(len(strArr))
		} else {
			num = ASSETSLOGPERNUM
		}
		dblogic.R("STATISTICS", assetLogs.RdsFlag).LTrim(key, 0, -num-1) //从队列中删除
		assetLogs.strToDb(strArr)                                        //批量入库
		time.Sleep(time.Millisecond * 200)
	}
}

// 流水记录入库
func (assetLogs *AssetsLogs) strToDb(strArr []string) bool {
	tb := &tables.Assetslog{}
	sqlArr := map[string][]tables.Assetslog{}
	for _, val := range strArr {
		err := json.Unmarshal([]byte(val), &tb)
		if tb.GameVersion == 0 {
			tb.GameVersion = 1
		}
		if err != nil {
			log.LogError("async_assets_log", "json unmarshal log str error", err, val)
			continue
		}
		tName := GetAssetsLogTableName(tb.TableName(), tb.Uid)
		if _, ok := sqlArr[tName]; !ok {
			sqlArr[tName] = []tables.Assetslog{*tb}
		} else {
			sqlArr[tName] = append(sqlArr[tName], *tb)
		}
	}
	for name, val := range sqlArr {
		err := dblogic.M("LOG").Table(name).Create(val).Error
		//批量插入,重复则忽略
		//db.Clauses(clause.Insert{Modifier: "IGNORE"}).Create(&user)
		if err != nil {
			log.LogError("async_assets_log", "multi insert int assets_log error", err)
		}
	}
	return true
}
