package statistics

import (
	"math/rand"
	"strconv"
	"time"

	jsoniter "github.com/json-iterator/go"
	"github.com/json-iterator/go/extra"
)

// json数据类型兼容php的字符类型的数值
var json = jsoniter.ConfigCompatibleWithStandardLibrary

const (
	DAYFORMAT      = "20060102"
	DATEFORMAT     = "2006-01-02"
	DAYHOURFORMAT  = "2006010215"
	DATETIMEFORMAT = "2006-01-02 15:04:05"
)

const (
	ANDROIDSID = 1 //andorid sid
	IOSSID     = 2 //ios sid
)

type CommonCls struct {
	Friend  *Friend  //好友
	Payment *Payment //支付
	Member  *Member  //
}

var (
	CstZone = time.FixedZone("CST", 0) //设置时区

	CrazyCls *CommonCls //初始化一个不需要重复实例化的实例类列表
)

func init() {
	//兼容php json string(float/int)等类型
	extra.RegisterFuzzyDecoders()
	CrazyCls = NewCommonCls()
	time.Local = CstZone
}

func NewCommonCls() *CommonCls {
	return &CommonCls{
		Payment: NewPayment(),
		Friend:  NewFirend(),
		Member:  NewMember(),
	}
}

// 获取当前时间之前一个小时(到小时)
func GetDayAfterHour() string {
	m, _ := time.ParseDuration("-1h")
	ret := time.Now().Add(m).In(time.FixedZone("CST", 0)).Format(DAYHOURFORMAT)
	return ret
}

// 获取当前时间(到小时)流水都是写零时区时间
func GetDayHour() string {
	return time.Now().In(time.FixedZone("CST", 0)).Format(DAYHOURFORMAT)
	//return time.Now().Format(DAYHOURFORMAT)
}

// 获取当天日期(例如:20210301)
func GetYmdDay() string {
	return time.Now().Format(DAYFORMAT)
}

// 获取当天日期(例如:2021-03-31)
func GetYmdDate() string {
	return time.Now().Format(DATEFORMAT)
}

// 获取n天以前的日期
func GetDaysBeforeDate(n int) string {
	return time.Now().AddDate(0, 0, -n).Format(DATEFORMAT)
}

// 获取当前时间(到秒)
func GetDateTime() string {
	return time.Now().Format(DATETIMEFORMAT)
}

// 获取分表的表名
func GetCircleTableName(table string, circleId int64) string {
	return table + strconv.Itoa(int(circleId)%100)
}

// 获取分表的表名
func GetAssetsLogTableName(table string, circleId int64) string {
	return table + "_" + time.Now().Format(DAYFORMAT) + "_" + strconv.Itoa(int(circleId)%10)
}

// 设置随机过期时间
func RedisRandomExpire(expire int64) time.Duration {
	return time.Second*time.Duration(expire) + time.Second*time.Duration(rand.Intn(300))
}
