package statistics

import (
    "time"
    "strings"
    "reflect"
    "strconv"

    "Crazy_Push/models/log"
    "Crazy_Push/models/tables"
    "Crazy_Push/models/dblogic"
    "Crazy_Push/models/okeys"
    "Crazy_Push/core/function"
)

const (
    NEWGUIDE            = "10001"  //新手引导
    NEWVERSIONGUIDE     = "10005" //新版新手引导
    CREATEROLE          = "10006" //创建角色
    NEWGUIDESLOT        = "10007" //新手引导，老虎机
    PAYREPORT           = "10008" //支付上报
    PAYREPORTSUCCESS    = "10009" //支付上报支付成功
    PAYREPORTCANCEL     = "10010" //支付上报取消支付
    PAYFAIL             = "10011"
    ADVERTISEID         = "10012" //广告Id
    NETWORKCONNECTERROR = "10013" //网络异常
    LOGINEVENT          = "10014"
    GIFTEVENT           = "10015"
    INVITEEVENT         = "10016"
    SHAREEVENT          = "10017"
    ACTIVITYEVENT       = "10018"
    COIZNER             = "10019"
    CALLBACKFRIEND      = "10020"

    LOGINSTART          = "100001"    //游戏启动
    LOGINGUESTCLICK     = "100002" //点击“游客”
    LOGINFBLOGIN        = "100003" //“fb”原生登录
    LOGINFBCANCEL       = "100004" //“fb”原生登录失败
    LOGINFBFAIL         = "100005" //“fb”原生登录失败
    LOGINFBSUCC         = "100006" //“fb”原生登录成功
    LOGINFBCLICK        = "100007" //点击“FB”
    LOGINAPPLELOGIN     = "100008" //"apple"原生登录
    LOGINAPPLEFAIL      = "100009" //“apple”原生登录失败
    LOGINAPPLESUCC      = "100010" //“apple”原生登录成功
    LOGINAPPLECLICK     = "100011" //点击“apple”登录
    LOGINSRVREQUEST     = "100012" //请求登录Srv
    LOGINSRVSUCC        = "100013" //登录Srv成功
    LOGINSRVTIMEOUT     = "100014" //登录Srv超时
    LOGINSRVFAIL        = "100015" //登录Srv超时
    FBCONNECT           = "100016" //登录后玩家绑定Facebook
    GUELOGSUC           = "100017" //游客登录成功
    GUEREGSUC           = "100018" //游客注册成功
    FBLOGSUC            = "100019" //FB登录成功
    FBREGSUC            = "100020" //FB重试登录成功
    APPLELOGSUC         = "100021" //苹果登录成功
    APPLEREGSUC         = "100022" //苹果重试登录成功
    ENTERMAINSCENE      = "100023" //转景到MainScene
    LOGINGUSESTCLICK    = "login_gusest_click"
    INVITEFRIEND        = "invite_friend"
    INVITEFRIENDSUCCESS = "invite_friend_success"

    ADCLICKED           = "10002"
    ADPLAY              = "10003"
)

var buyEvents = map[string]string {
    PAYREPORT:        "pay_report",
    PAYREPORTSUCCESS: "pay_report_success",
    PAYREPORTCANCEL:  "pay_report_cancel",
    PAYFAIL:          "pay_fail",
}

var userEvents = map[string]string{
    ADCLICKED:  "ad_clicked",
    ADPLAY:      "ad_play",
    //ADVERTISEID: "advertiseID", //广告Id
}

//只记录漏斗的上报事件
var funnelEvents = map[string]string {
    LOGINSTART:      "login_start",       //游戏启动
    LOGINGUESTCLICK: "login_guest_click", //点击“游客”
    LOGINFBLOGIN:    "login_fb_login",    //“fb”原生登录
    LOGINFBCANCEL:   "login_fb_cancel",   //“fb”原生登录失败
    LOGINFBFAIL:     "login_fb_fail",     //“fb”原生登录失败
    LOGINFBSUCC:     "login_fb_succ",     //“fb”原生登录成功
    LOGINFBCLICK:    "login_fb_click",    //点击“FB”
    LOGINAPPLELOGIN: "login_apple_login", //"apple"原生登录
    LOGINAPPLEFAIL:  "login_apple_fail",  //“apple”原生登录失败
    LOGINAPPLESUCC:  "login_apple_succ",  //“apple”原生登录成功
    LOGINAPPLECLICK: "login_apple_click", //点击“apple”登录
    LOGINSRVREQUEST: "login_srv_request", //请求登录Srv
    LOGINSRVSUCC:    "login_srv_succ",    //登录Srv成功
    LOGINSRVTIMEOUT: "login_srv_timeout", //登录Srv超时
    LOGINSRVFAIL:    "login_srv_fail",    //登录Srv超时
    FBCONNECT:       "fb_connect",        //登录后玩家绑定Facebook
    GUELOGSUC:       "gue_logsuc",        //游客登录成功
    GUEREGSUC:       "gue_regsuc",        //游客注册成功
    FBLOGSUC:        "fb_logsuc",         //FB登录成功
    FBREGSUC:        "fb_regsuc",         //FB重试登录成功
    APPLELOGSUC:     "apple_logsuc",      //苹果登录成功
    APPLEREGSUC:     "apple_regsuc",      //苹果重试登录成功
    ENTERMAINSCENE:  "enter_mainscene",   //转景到MainScene
}

//所有上报事件
var allReportEvents = map[string]string {
    NEWGUIDE:            "new_guide",             //新手引导
    NEWVERSIONGUIDE:     "new_version_guide",     //新版新手引导
    CREATEROLE:          "create_role",           //创建角色
    NEWGUIDESLOT:        "new_guide_slot",        //新手引导，老虎机
    PAYREPORT:           "pay_report",            //支付上报
    PAYREPORTSUCCESS:    "pay_report_success",     //支付上报支付成功
    PAYREPORTCANCEL:     "pay_report_cancel",     //支付上报取消支付
    ADVERTISEID:         "advertiseID",           //广告Id
    NETWORKCONNECTERROR: "network_connect_error", //网络异常
    LOGINEVENT:          "login_event",
    GIFTEVENT:           "gift_event",
    INVITEEVENT:         "invite_event",
    SHAREEVENT:          "share_event",
    ACTIVITYEVENT:       "activity_event",
    COIZNER:             "coizner",
    CALLBACKFRIEND:      "callback_friend",

    LOGINSTART:          "login_start",           //游戏启动
    LOGINGUESTCLICK:     "login_guest_click",     //点击“游客”
    LOGINFBLOGIN:        "login_fb_login",        //“fb”原生登录
    LOGINFBCANCEL:       "login_fb_cancel",       //“fb”原生登录失败
    LOGINFBFAIL:         "login_fb_fail",         //“fb”原生登录失败
    LOGINFBSUCC:         "login_fb_succ",         //“fb”原生登录成功
    LOGINFBCLICK:        "login_fb_click",        //点击“FB”
    LOGINAPPLELOGIN:     "login_apple_login",     //"apple"原生登录
    LOGINAPPLEFAIL:      "login_apple_fail",      //“apple”原生登录失败
    LOGINAPPLESUCC:      "login_apple_succ",      //“apple”原生登录成功
    LOGINAPPLECLICK:     "login_apple_click",     //点击“apple”登录
    LOGINSRVREQUEST:     "login_srv_request",     //请求登录Srv
    LOGINSRVSUCC:        "login_srv_succ",        //登录Srv成功
    LOGINSRVTIMEOUT:     "login_srv_timeout",     //登录Srv超时
    LOGINSRVFAIL:        "login_srv_fail",        //登录Srv超时
    FBCONNECT:           "fb_connect",            //登录后玩家绑定Facebook
    GUELOGSUC:           "gue_logsuc",            //游客登录成功
    GUEREGSUC:           "gue_regsuc",            //游客注册成功
    FBLOGSUC:            "fb_logsuc",             //FB登录成功
    FBREGSUC:            "fb_regsuc",             //FB重试登录成功
    APPLELOGSUC:         "apple_logsuc",          //苹果登录成功
    APPLEREGSUC:         "apple_regsuc",          //苹果重试登录成功
    ENTERMAINSCENE:      "enter_mainscene",       //转景到MainScene

    LOGINGUSESTCLICK:    "login_gusest_click",
    INVITEFRIEND:        "invite_friend",
    INVITEFRIENDSUCCESS: "invite_friend_success",
}

type RcommonData struct {
    RegisterTime int64 `json:register_time`
    TotalScore   int64 `json:total_score`
    Level        int64 `json:level`
    Star         int64 `json:star`
    Spin         int64 `json:spin`
    Coin         int64 `json:coin`
    Building     int64 `json:building`
    IsFix        int64 `json:is_fix`
    IsGuest     string `json:is_guest`
}

type EventReport struct {}

func NewEventReport() *EventReport {
    return &EventReport{}
}

//是否漏斗打点事件
func (rep *EventReport)IsFunnelReport(eventId string) bool {
    if _, ok := funnelEvents[eventId]; !ok {
        return false
    }
    return true
}

//是否带uid的埋点事件
func (rep *EventReport)IsUserReport(eventId string) bool {
    if _, ok := userEvents[eventId]; !ok {
        return false
    }
    return true
}

//是否支付上报事件
func (rep *EventReport)IsBuyReport(eventId string) bool{
    if _, ok := buyEvents[eventId]; !ok {
        return false
    }
    return true
}

func (rep *EventReport)GetTypeByFunnel(eventId string) string {
    if _, ok := allReportEvents[eventId]; !ok {
        return ""
    }
    return allReportEvents[eventId]
}

/**
 * 埋点上报到redis队列,go上报到elk
 * @param array param
 */
func (rep *EventReport)FunnelReport(param map[string]interface{}, ip string){
    keys := rep.GetTypeByFunnel(param["event_id"].(string))
    if keys == "" {
        return
    }

    funnelData := map[string]string{
        "tag":      "bury-point",
        "topic":    keys,
        "time":     param["serverTime"].(string),
        "event_id": param["event_id"].(string),
        "content":  param["data"].(string),
        "uid":      param["uid"].(string),
        "client_ip": ip,
    }
    resStr, _ := json.Marshal(funnelData)

    dblogic.R("Statistics").LPush(okeys.Funnel(), resStr)
}

//将上报事件data转化成数组
func (rep *EventReport) ParseData(retStr string) []string {
    var data []interface{}
    var ret = []string{}
    var stepStr string
    //直接替换
    if strings.Contains(retStr, "[[") && strings.Contains(retStr, "]]") {
        retStr = strings.Replace(retStr, "[[", "[", -1)
        retStr = strings.Replace(retStr, "]]", "]", -1)
    }
    if strings.Contains(retStr, "[") {
        err := json.Unmarshal([]byte(retStr), &data)
        if err != nil {
            log.LogError("event_report_log", "json unmarshal error", retStr)
            return ret
        }
        for _, v := range data {
            if v == nil {
                //log.LogError("event_report_log", "reflect type with nil", retStr)
                ret = append(ret, "")
                continue
            }
            kind := reflect.TypeOf(v).Kind()
            if kind == reflect.String {
                stepStr = v.(string)
                ret = append(ret, stepStr)
            } else if kind == reflect.Int64 {
                stepStr = strconv.Itoa(int(v.(int64)))
                ret = append(ret, stepStr)
            } else if kind == reflect.Float64 {
                stepStr = strconv.Itoa(int(v.(float64)))
                ret = append(ret, stepStr)
            } else {
                //log.LogError("event_report_log", "reflect type with nil", retStr)
                //ret = append(ret, "")
            }
        }
    } else {
        stepStr = retStr
        ret = append(ret, retStr)
    }
    return ret
}

//埋点事件处理
func (rep *EventReport) BuryingPoint(eventId string, uid string, data []string, commonData string) {
    eventIdInt64, _ := strconv.ParseInt(eventId, 10, 64)
    uidInt64, _ := strconv.ParseInt(uid, 10, 64)
    var cData *RcommonData
    err := json.Unmarshal([]byte(commonData), &cData)
    if err != nil {
        return
    }
    var isGuest int64 = 0
    if strings.ToUpper(cData.IsGuest) == "FACEBOOK" {
        isGuest = 1
    }
    pushData := map[string]interface{}{
        "uid":            uidInt64,
        "event_id":       eventIdInt64,
        "is_guest":       isGuest,
        "register_time":  cData.RegisterTime,
        "total_recharge": 0,
        "total_score":    cData.TotalScore,
        "first_pay_time": 0,
        "last_pay_time":  0,
        "friend_num":     0,
        "ad_channel":     0,
        "ad_type":        0,
        "level":          cData.Level,
        "star":           cData.Star,
        "spins":          cData.Spin,
        "money":          cData.Coin,
        "building":       cData.Building,
        "is_fix":         cData.IsFix,
        "time":           time.Now().Unix(),
    }

    for i:= 0; i <= 19; i++ {
        iStr := strconv.Itoa(i)
        if len(data) < i + 1 {
            pushData["z"+iStr] = 0
        }else{
            pushData["z"+iStr] = rep.ParseZ(data[i])
        }
    }

    pushData["total_recharge"] = CrazyCls.Payment.GetUserTotalPayNum(uid, 0) //充值总额
    pushData["first_pay_time"] = CrazyCls.Payment.GetUserFirstPayTime(uid)   //首次充值时间
    pushData["last_pay_time"]  = CrazyCls.Payment.GetUserLastPayTime(uid)    //最后充值时间
    pushData["friend_num"]     = len(CrazyCls.Friend.GetUserAllFriend(uid, "all")) //好友数量
    pushData["ad_type"]        = rep.GetUserAdjustGroup(uid)    //广告类型
    pushData["ad_channel"]     = rep.GetUserAdjustChannel(uid)  //广告渠道

    //if (eventId == "order_finish" || eventId == PAYREPORTSUCCESS) && len(data) >= 19 {
    if eventId == PAYREPORTSUCCESS && len(data) >= 19 {
        aid, err2 := strconv.ParseInt(data[18], 10, 64)
        if err2 != nil { //数据错误
            return
        }
        pushData["z14"] = CrazyCls.Payment.GetUserPayTimes(uid, 0) //是第几次完成充值
        //收集 pushData值(第几套活动数值)
        if data[18] == "" {
            pushData["z15"] = 0
        } else {
            rid, err2 := dblogic.R("games").Get(okeys.UserBindActRid(uidInt64, aid))
            if err2 != nil {
                return
            }
            if rid == "" {
                pushData["z15"] = 0
            } else {
                pushData["z15"], err2 = strconv.ParseInt(rid, 10, 64)
                if err2 != nil {
                    return
                }
            }
        }
        pushData["z16"] = CrazyCls.Payment.GetUserUpPayTime(uid) //上次支付时间
        pushData["z17"] = CrazyCls.Member.GetUserLoginTime(uid) //当前累计游戏时长
    }

    day, _ := strconv.Atoi(function.Date("20060102", time.Now().Unix()))
    key := okeys.BuryingPointLog(int64(day))
    pushStr, _ := json.Marshal(pushData)

    dblogic.R("Statistics").LPush(key, pushStr) //流水入队
}

//字符串转int64
func (rep *EventReport) ParseZ(str string) int64 {
    z, err := strconv.ParseInt(str, 10, 64)
    if err != nil {
        return 0
    }
    return z
}

//获取广告数据
func (rep *EventReport) AdJustData(uid int64) (*tables.TAdjustSta, *tables.TAdjustData) {
    adSta := &tables.TAdjustSta{}
    tname := GetCircleTableName(adSta.TableName(), uid)

    dblogic.M("ADJUSTSTA").Table(tname).Where("uid = ?", uid).First(&adSta)
    if (adSta.Ios == "" && adSta.Android == "") || (adSta.Ios != "" && adSta.Android != "") {
        return adSta, nil
    }
    adData := &tables.TAdjustData{}
    agps := strings.Trim(adSta.Android, "\"")
    igps := strings.Trim(adSta.Ios, "\"")
    err := dblogic.M("ADJUSTSTA").Where("gps_adid = ? OR idfa = ?", agps, igps).First(&adData).Error
    if err != nil {
        return adSta, nil
    }
    return adSta, adData
}

//返回广告渠道
func (rep *EventReport) GetUserAdjustChannel(uidStr string) string {
    uid, _ := strconv.ParseInt(uidStr, 10, 64)
    _, adData := rep.AdJustData(uid)
    if adData == nil {
        return ""
    }
    return adData.NetworkName
}

//返回玩家所在广告分组
func (rep *EventReport) GetUserAdjustGroup(uidStr string) string {
    uid, _ := strconv.ParseInt(uidStr, 10, 64)
    cacheKey   := okeys.UserAdjustGroup(uid)
    cache, err := dblogic.R("Games").Get(cacheKey)
    if  err != nil {
        return ""
    }
    if cache != "" {
        return cache
    }
    var group string = ""
    _, adData := rep.AdJustData(uid)
    if adData == nil {
        return "Not"
    }
    switch adData.CampaignName {
        case "UAC2.0", "UAC2.5", "AEO", "VO", "Recall":
            group = adData.CampaignName
        default:
            group = "Not"
    }
    //3天过期
    dblogic.R("Games").SetXX(cacheKey, group, RedisRandomExpire(3 * 86400))
    return group
}
