package statistics

import (
    "fmt"
    "time"
    "strconv"

    "Crazy_Push/models/log"
    "Crazy_Push/models/tables"
    "Crazy_Push/models/dblogic"
    "Crazy_Push/models/okeys"
)

const (
    FIRSTPAYKEYFLAG = 1 //首次购买key标志
    LASTPAYKEYFLAG  = 2 //最后一次购买key标志
)

type Payment struct { }

func NewPayment() *Payment {
    return &Payment{}
}

/**
 * 用戶最後購買時間
 * @param uid
 * @return int64
 */
func (payment *Payment) GetUserFirstPayTime(uid string) int64 {
    return payment.getUserFirstOrLastPayTime(uid, LASTPAYKEYFLAG)
}

//充值总额
func (payment *Payment) GetUserTotalPayNum(uid string, day int64) float64 {
    uidInt64, _ := strconv.ParseInt(uid, 10, 64)
    cacheKey   := okeys.UserTotalPayNum(uidInt64)
    cache, err := dblogic.R("games").HGet(cacheKey, fmt.Sprintf("%d", day))
    if err == nil && cache != "" {
        sum, err := strconv.ParseFloat(cache, 64)
        if err != nil {
            return 0
        }
        return sum
    }
    var payNum float64
    tb := &tables.Payment{}
    tname := GetCircleTableName(tb.TableName(), uidInt64)
    if day > 0 {
        stamp := time.Now().Unix() - day * 24 * 60 * 60
        dblogic.M("PAYMENT").Table(tname).Select([]string{"COALESCE(SUM(pusd),0) AS sum"}).Where("uid = ? AND status = 2 AND pay_time > ?", uid, stamp).Scan(&payNum)
    }else{
        dblogic.M("PAYMENT").Table(tname).Select([]string{"COALESCE(SUM(pusd),0) AS sum"}).Where("uid = ? AND status = 2", uid).Scan(&payNum)
    }

    if payNum <= 0 {
        return 0
    }

    sum, _ := strconv.ParseFloat(fmt.Sprintf("%.2f", payNum), 64)
    dblogic.R("Games").HSet(cacheKey, fmt.Sprintf("%d", day), sum)
    dblogic.R("Games").Expire(cacheKey, RedisRandomExpire(7 * 86400))
    return sum
}

//最后充值时间
func (payment *Payment) GetUserLastPayTime(uid string) int64 {
    return payment.getUserFirstOrLastPayTime(uid, LASTPAYKEYFLAG)
}

//获取玩家首次或最后一次充值时间
func (payment *Payment) getUserFirstOrLastPayTime(uid string, flag int64) int64 {
    var cacheKey string
    uidInt64, _ := strconv.ParseInt(uid, 10, 64)
    if flag == FIRSTPAYKEYFLAG {
        cacheKey = okeys.UserFirstPayTime(uidInt64)
    } else {
        cacheKey = okeys.UserLastPayTime(uidInt64)
    }
    cache, err := dblogic.R("Games").Get(cacheKey)
    if err != nil {
        log.LogError("static_payment", "get user pay time error", uid, flag, err, cache)
        return 0
    }

    if cache != "" {
        num, _ := strconv.ParseInt(cache, 10, 64)
        return num
    }

    info := payment.getUserFirstOrLastPayInfo(uid, flag)
    return info.PayTime
}

//获取玩家首次或最后一次支付信息
func (payment *Payment) getUserFirstOrLastPayInfo(uid string, flag int64) *tables.Payment {
    uidInt64, _ := strconv.ParseInt(uid, 10, 64)
    var orderKey string
    if flag == FIRSTPAYKEYFLAG {
        orderKey = "pid ASC"
    } else {
        orderKey = "pid DESC"
    }

    tb := &tables.Payment{}
    tname := GetCircleTableName(tb.TableName(), uidInt64)
    dblogic.M("PAYMENT").Table(tname).
        Where("uid = ? AND status = 2", uid).
        Order(orderKey).
        Order("pid DESC").First(&tb)
    return tb
}

//第几次充值
func (payment *Payment) GetUserPayTimes(uid string, day int64) int64 {
    uidInt64, _ := strconv.ParseInt(uid, 10, 64)
    cacheKey := okeys.UserPayTimes(uidInt64)
    cache, err := dblogic.R("games").HGet(cacheKey, strconv.FormatInt(day, 10))
    if err != nil {
        log.LogError("static_payment", "get user pay times error", uid, day, cacheKey, err, cache)
    }
    if cache != "" {
        num, _ := strconv.ParseInt(cache, 10, 64)
        return num
    }

    tb := &tables.Payment{}
    tname := GetCircleTableName(tb.TableName(), uidInt64)
    var payNum int64
    if day > 0 {
        stamp := time.Now().Unix() - day * 24 * 60 * 60
        dblogic.M("PAYMENT").Table(tname).Select([]string{"COUNT(*) AS count"}).Where("uid = ? AND status = 2 AND pay_time > ?", uid, stamp).Scan(&payNum)
    } else {
        dblogic.M("PAYMENT").Table(tname).Select([]string{"COUNT(*) AS count"}).Where("uid = ? AND status = 2", uid).Scan(&payNum)
    }
    dblogic.R("Games").HSet(cacheKey, fmt.Sprintf("%d", day), payNum)
    dblogic.R("Games").Expire(cacheKey, RedisRandomExpire(7 * 86400))
    return payNum
}

//上次支付时间(last前一次)
func (payment *Payment) GetUserUpPayTime(uid string) int64 {
    uidInt64, _ := strconv.ParseInt(uid, 10, 64)
    tb          := &tables.Payment{}
    tname       := GetCircleTableName(tb.TableName(), uidInt64)

    err := dblogic.M("PAYMENT").Table(tname).
        Where("uid = ? AND status = 2", uid).
        Limit(1).
        Offset(1).
        Order("pid DESC").First(&tb).Error

    if err != nil {
        return 0
    }
    return tb.PayTime
}
