package ta

import (
	"Crazy_Push/core/exp"
	//"encoding/json"
	"fmt"
	"os"
	"reflect"
	"strconv"
	//"sync"
	"time"

	"Crazy_Push/configs"
	"Crazy_Push/models/dblogic"

	"github.com/ThinkingDataAnalytics/go-sdk/thinkingdata"
	"github.com/go-redis/redis"
	jsoniter "github.com/json-iterator/go"
)

const (
	TAPERNUM   = 5000  //普通消息,每次从队列中取出的数量
	SLOTPERNUM = 20000 //老虎机消息
	SLOTEVENT  = "slot_machine"

	ADSCOSTAPPID = "999"
)

var (
	json          = jsoniter.ConfigCompatibleWithStandardLibrary
	appId         = sysConfig.GetTaAppId()
	serverUrl     = sysConfig.GetTaServer()
	trackDataChan = map[string]chan *TrackData{}
	flushChan     = make(chan int, 50) // 触发上报.
	interrupt     = make(chan os.Signal, 1)
)

type TrackData struct {
	Time       string `json:"#time"`
	AccountId  uint64 `json:accountId`
	DistinctId string `json:distinctId`
	//Type string `json:type,omitempty`
	Type       string                 `json:type`
	EventName  string                 `json:eventName`
	Properties map[string]interface{} `json:properties`
}

type Ta struct {
	TaObj thinkingdata.TDAnalytics
	//WaitGroup *sync.WaitGroup
}

func SetChan(event []string) {
	for _, x := range event {
		trackDataChan[x] = make(chan *TrackData, 20)
	}
}

func NewTa() *Ta {
	consumer, _ := thinkingdata.NewBatchConsumerWithBatchSize(serverUrl, appId, 200)
	ta := thinkingdata.New(consumer)
	return &Ta{
		TaObj: ta,
		//WaitGroup: &sync.WaitGroup{},
	}
}

// 调度
func (t *Ta) TaDispatch(eventName string) {
	for {
		select {
		case trackData := <-trackDataChan[eventName]:
			//t.WaitGroup.Add(1)
			go t.ActionPush(trackData)
		case <-flushChan:
			t.TaObj.Flush()
		case <-interrupt:
			close(trackDataChan[eventName])
			close(interrupt)
			//t.WaitGroup.Wait()
			return //退出
		}
	}
}

func (t *Ta) transformString(k interface{}) string {
	switch reflect.TypeOf(k).Kind() {
	case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64, reflect.Uint, reflect.Uint8, reflect.Uint16, reflect.Uint32, reflect.Uint64:
		return strconv.FormatUint(k.(uint64), 10)
	case reflect.Float32, reflect.Float64:
		return strconv.FormatFloat(k.(float64), 'f', 10, 64)
	default:
		return k.(string)
	}
}

func (t *Ta) ActionPush(trackData *TrackData) {
	//fmt.Println("Action Event:"+trackData.EventName)
	//defer t.WaitGroup.Done()
	var err error
	newProperties := map[string]interface{}{}
	var cur interface{}
	for i, x := range trackData.Properties {
		//fmt.Println("Action Properties:"+i)
		//fmt.Println(x)
		if x == nil {
			cur = 0
		} else {
			if reflect.TypeOf(x).Kind() == reflect.Slice {
				a := x.([]interface{})
				s := []string{}
				for _, k := range a {
					exp.Try(func() {
						one := t.transformString(k)
						s = append(s, one)
					},
						func(ex exp.Exception) {
							fmt.Printf("事件:%+v,数据格式异常 Err:%+v", trackData.EventName, ex.Err)
						},
					)

				}
				cur = s
			} else {
				cur = x
			}
		}
		newProperties[i] = cur
	}
	trackData.Properties = newProperties
	timeObj, err := time.Parse("2006-01-02 15:04:05", trackData.Time)
	if err == nil {
		dateTs := timeObj.In(time.FixedZone("CST", 8*3600)).Format("2006-01-02 15:04:05")
		trackData.Properties["#time"] = dateTs
	}

	accountId := strconv.FormatUint(trackData.AccountId, 10)
	if accountId == ADSCOSTAPPID {
		accountId = "admin"
	}
	if trackData.EventName == "user_setOnce" {
		err = t.TaObj.UserSetOnce(accountId, trackData.DistinctId, trackData.Properties)
	} else if trackData.EventName == "user_set" {
		err = t.TaObj.UserSet(accountId, trackData.DistinctId, trackData.Properties)
		//} else if trackData.Type == "track_update" {
		//	err = t.TaObj.TrackUpdate(accountId,trackData.DistinctId,trackData.Properties)
	} else {
		err = t.TaObj.Track(accountId, trackData.DistinctId, trackData.EventName, trackData.Properties)
	}
	if err != nil {
		fmt.Println("Track Push Err:" + err.Error())
	} else {
		//fmt.Println("Push:" + trackData.EventName)
	}
	//fmt.Println("Finish Push:" + trackData.EventName)
	//t.WaitGroup.Done()
}

// 批量出队
func (t *Ta) PopDataFromRedis(eventName string) {
	//tt, tt2 := time.Now(), time.Now()
	strArr, err := dblogic.R("STATISTICS", eventName).LRange("SS_EVENT:"+eventName, 0, TAPERNUM-1)
	if err != nil {
		fmt.Println("取redis数据出错:", err.Error())
		dblogic.CloseRd("STATISTICS", eventName) //删除失败的连接,下次重新连接
		//panic(err.Error())
		return
	}
	if len(strArr) == 0 {
		time.Sleep(time.Millisecond * 50)
		return
	}
	var num, cunsumeNum int64
	if eventName == SLOTEVENT {
		cunsumeNum = SLOTPERNUM
	} else {
		cunsumeNum = TAPERNUM
	}
	if len(strArr) < int(cunsumeNum) {
		num = int64(len(strArr))
	} else {
		num = cunsumeNum
	}

	_, err = dblogic.R("STATISTICS", eventName).LTrim("SS_EVENT:"+eventName, num, -1) //从队列中删除
	if err != nil && err != redis.Nil {
		fmt.Println(eventName + ":获取数数事件异常,redis读取失败")
		os.Exit(-1)
	}
	fmt.Println(eventName+":获取条数:", num)
	//elapsed := time.Since(tt)
	//tt = time.Now()
	//fmt.Println("取数据消耗时间:", elapsed,",获取条数:", num)
	for _, str := range strArr {
		trackData := &TrackData{}
		err = json.Unmarshal([]byte(str), trackData)
		if err != nil {
			fmt.Println("Redis UnMarSha1 Err:" + err.Error())
			continue
		}
		if trackData.EventName == "" {
			continue
		}
		if trackDataChan[trackData.EventName] == nil {
			continue
		}
		//fmt.Println(trackData.EventName)
		trackDataChan[trackData.EventName] <- trackData
	}
	flushChan <- 1 //触发一次上报
	if num <= 50 { //数据太少,休息200毫秒
		time.Sleep(time.Millisecond * 50)
	}
	//elapsed = time.Since(tt)
	//fmt.Println("写数据消耗时间:", elapsed)
	//tt = time.Now()
	////elapsed = time.Since(tt)
	//elapsed2 := time.Since(tt2)
	////fmt.Println("消费消耗时间:", elapsed)
	//fmt.Println(eventName+":总共消耗时间:", elapsed2)
}
