package log

import (
	"fmt"
	"sync/atomic"
)

const (
	Debug = "Debug"
	Info  = "Info"
	Warn  = "Warn"
	Error = "Error"
	Panic = "Panic"
)
const (
	DebugLv Level = iota - 1
	InfoLv
	WarnLv
	ErrorLv
	PanicLv
)

var levelMap = map[Level]string{
	DebugLv: Debug,
	InfoLv:  Info,
	WarnLv:  Warn,
	ErrorLv: Error,
	PanicLv: Panic,
}

var levelStrMap = map[string]Level{
	Debug: DebugLv,
	Info:  InfoLv,
	Warn:  WarnLv,
	Error: ErrorLv,
	Panic: PanicLv,
}

type Level int8

func (l Level) String() string {
	str, exist := levelMap[l]
	if !exist {
		str = fmt.Sprintf("Level(%d)", l)
	}
	return str
}

func (l *Level) Parse(str string) {
	lv, exist := levelStrMap[str]
	if !exist {
		lv = InfoLv
	}
	*l = lv
}

func NewAtomicLevel() AtomicLevel {
	return AtomicLevel{
		lv: int32(InfoLv),
	}
}

type AtomicLevel struct {
	lv int32
}

func (a *AtomicLevel) Level() Level {
	return Level(atomic.LoadInt32(&a.lv))
}

func (a *AtomicLevel) SetLevel(lv Level) {
	atomic.StoreInt32(&a.lv, int32(lv))
}

func (a *AtomicLevel) SetLevelWithStr(lvStr string) {
	var lv Level
	lv.Parse(lvStr)
	a.SetLevel(lv)
}

func (a *AtomicLevel) Open(lv Level) bool {
	return lv >= a.Level()
}
